/*************************************************************************
 *
 *  $RCSfile: binarybasereader.hxx,v $
 *
 *  $Revision: 1.2 $
 *
 *  last change: $Author: lla $ $Date: 2001/05/31 14:10:02 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef CONFIGMGR_BINARYBASEREADER_HXX
#define CONFIGMGR_BINARYBASEREADER_HXX

#include <memory>
#include <rtl/ustring.hxx>
#include <com/sun/star/lang/XMultiServiceFactory.hpp>

#include <com/sun/star/io/IOException.hpp>

#include <com/sun/star/io/XDataInputStream.hpp>
#include <com/sun/star/io/XMarkableStream.hpp>

namespace configmgr
{
	namespace css = com::sun::star;
	namespace uno = css::uno;
	namespace io  = css::io;
	namespace lang = css::lang;

// -----------------------------------------------------------------------------

	class OInputMark;

class OBinaryBaseReader 
{
	rtl::OUString  m_sFileURL;
	uno::Reference<lang::XMultiServiceFactory> m_xServiceProvider;

	uno::Reference<io::XDataInputStream> m_xDataInputStream;
	uno::Reference<io::XMarkableStream>  m_xMarkableStream;

public:
	explicit OBinaryBaseReader(rtl::OUString const& _sFileURL, uno::Reference< lang::XMultiServiceFactory > const& _rxServiceProvider)
			:m_sFileURL(_sFileURL),
			 m_xServiceProvider(_rxServiceProvider)
		{}
	void open();
	void dispose();
	~OBinaryBaseReader();

	void read(sal_Bool &_nValue) throw (io::IOException);
	void read(sal_Int8 &_nValue) throw (io::IOException);
	void read(sal_Int16 &_nValue) throw (io::IOException);
	void read(sal_Int32 &_nValue) throw (io::IOException);
	void read(sal_Int64 &_nValue) throw (io::IOException);
	void read(double &_nValue) throw (io::IOException);
//!	void read(sal_Unicode &pStr) throw (io::IOException);
	void read(sal_Unicode &_sValue) throw (io::IOException);
	void read(sal_Unicode *pStr, sal_Int32 _nLength) throw (io::IOException);
	void read(uno::Sequence<sal_Int8> &_aBuf, sal_Int32 _nLength) throw (io::IOException);
	void read(rtl::OUString& _aStr) throw (io::IOException);
	void read(uno::Sequence<sal_Int8> &_aValue) throw (io::IOException); // binary

	uno::Reference<io::XMarkableStream>  getMarkableStream() {return m_xMarkableStream;}
	uno::Reference<io::XDataInputStream> getDataInputStream() {return m_xDataInputStream;}
	
private:
	void ensure_File() throw (com::sun::star::io::IOException);
};

// -----------------------------------------------------------------------------
class OInputMark
{
	sal_Int32 m_nMark;
	sal_Int64 m_nPosition;
	OBinaryBaseReader *m_pReader;
	uno::Reference<io::XMarkableStream> m_xMarkableStream;

	OInputMark(OInputMark const&);
	OInputMark& operator=(OInputMark const&);

public:
	OInputMark(OBinaryBaseReader* _pReader);
	void jumpToMark();
};
// -----------------------------------------------------------------------------
	void readSequenceValue(OBinaryBaseReader& _rReader, uno::Any &_aValue, uno::Type const& aElementType);


}

#endif
