/*************************************************************************
 *
 *  $RCSfile: updatetree.cxx,v $
 *
 *  $Revision: 1.10 $
 *
 *  last change: $Author: jb $ $Date: 2001/07/05 17:05:52 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <stdio.h> // include for solaris 8

#include "updatehandler.hxx"

#include <rtl/ustring.hxx>

#ifndef _OSL_FILE_HXX_
#include <osl/file.hxx>
#endif

#ifndef _COM_SUN_STAR_XML_SAX_XPARSER_HPP_
#include <com/sun/star/xml/sax/XParser.hpp>
#endif

#ifndef _COM_SUN_STAR_XML_SAX_SAXPARSEEXCEPTION_HPP_
#include <com/sun/star/xml/sax/SAXParseException.hpp>
#endif

#ifndef _COM_SUN_STAR_LANG_XMULTISERVICEFACTORY_HPP_
#include <com/sun/star/lang/XMultiServiceFactory.hpp>
#endif

#ifndef _COM_SUN_STAR_IO_XINPUTSTREAM_HPP_
#include <com/sun/star/io/XInputStream.hpp>
#endif

#ifndef _COM_SUN_STAR_IO_XOUTPUTSTREAM_HPP_
#include <com/sun/star/io/XOutputStream.hpp>
#endif

#ifndef _COM_SUN_STAR_XML_SAX_XEXTENDEDDOCUMENTHANDLER_HPP_
#include <com/sun/star/xml/sax/XExtendedDocumentHandler.hpp>
#endif

#ifndef _COM_SUN_STAR_IO_XACTIVEDATASOURCE_HPP_
#include <com/sun/star/io/XActiveDataSource.hpp>
#endif

#include "change.hxx"

#ifndef _CONFIGMGR_FILEHELPER_HXX_
#include "filehelper.hxx"
#endif

#ifndef _CONFIGMGR_OSLSTREAM_HXX_
#include "oslstream.hxx"
#endif

#include "xmlformater.hxx"

#define ASCII(x) rtl::OUString::createFromAscii(x)

namespace configmgr
{
	namespace updatetree
	{
		using namespace osl;
		namespace css = com::sun::star;
		namespace uno = css::uno;
		namespace sax = css::xml::sax;
		namespace io  = css::io;
		namespace lang = css::lang;

		// -----------------------------------------------------------------------------

		int doUpdateTree(const rtl::OUString& _aFromURL,
						 const rtl::OUString& _aToURL,
						 TreeChangeList& aChanges,
						 const uno::Reference< lang::XMultiServiceFactory >& _rxServiceProvider,
						 const vos::ORef<OOptions>& rOptions,
						 /* OUT */ rtl::OUString &sError)
		{
			
			sal_Int32 nRet = 0;
			
			// Filehandling
		{
			
			File aFile(_aFromURL);
			FileBase::RC eError = aFile.open(OpenFlag_Read);
			if (eError != osl_File_E_None)
			{
				sError = ASCII("doTreeUpdate: ") + FileHelper::createOSLErrorString(eError) + ASCII("\n with URL: ") + _aFromURL;
				
				rtl::OString aStr = rtl::OUStringToOString(sError,RTL_TEXTENCODING_ASCII_US);
				OSL_ENSURE(0, aStr.getStr());
				return 1;
			}
			
			File aWriteFile(_aToURL);
			eError = aWriteFile.open(OpenFlag_Write | OpenFlag_Create);
			// TODO: E_EXIST
			if (eError != osl_File_E_None)
			{
				if (eError == osl_File_E_EXIST)
				{
					FileHelper::tryToRemoveFile(_aToURL);
					eError = aWriteFile.open(OpenFlag_Write | OpenFlag_Create);
					if (eError != osl_File_E_None)
					{
						sError = ASCII("doTreeUpdate: ") + FileHelper::createOSLErrorString(eError) + ASCII("\n with URL: ") + _aToURL;
						
						rtl::OString aStr = rtl::OUStringToOString(sError,RTL_TEXTENCODING_ASCII_US);
						OSL_ENSURE(0, aStr.getStr());
						return 1;
					}
				}
			}
			
			// ------------- sax read --------------
			// create the writer and the parser we want to use

			uno::Reference< sax::XParser > xParser;
			xParser = uno::Reference< sax::XParser > (_rxServiceProvider->createInstance( ::rtl::OUString::createFromAscii("com.sun.star.xml.sax.Parser")), uno::UNO_QUERY);
		
			uno::Reference<io::XInputStream> xInputStream = new configmgr::OSLInputStreamWrapper(aFile);
		
			// connect stream to input stream to the parser
			sax::InputSource aInputSource;
			uno::Reference<io::XInputStream> xPipeInput( xInputStream, uno::UNO_QUERY );
			aInputSource.aInputStream = xPipeInput;
		
			// ------------- sax write -------------
		
			uno::Reference<io::XOutputStream> xOutputStream = new configmgr::OSLOutputStreamWrapper(aWriteFile);
		
			uno::Reference< sax::XDocumentHandler > xWriter;
			xWriter = uno::Reference< sax::XDocumentHandler > (_rxServiceProvider->createInstance( ::rtl::OUString::createFromAscii("com.sun.star.xml.sax.Writer")), uno::UNO_QUERY);
		
			uno::Reference< sax::XExtendedDocumentHandler> xHandler(xWriter, uno::UNO_QUERY);
			uno::Reference< io::XActiveDataSource> xDataSource( xHandler, uno::UNO_QUERY );
			xDataSource->setOutputStream(xOutputStream);

			OXMLAttributeHandler aAttributeHandler;
			aAttributeHandler.setCurrentModule( aChanges.getModuleName().toString() );

			XMLFormater aFormater(rOptions,aAttributeHandler);
			aFormater.setHandler(xWriter);

			uno::Reference<sax::XDocumentHandler> xMergeHandler = new TreeMerger(aChanges, aFormater);
			//xMergeHandler = new OValueConverter(xMergeHandler, false);

			xParser->setDocumentHandler( xMergeHandler );
			// parse
			try
			{
				xParser->parseStream( aInputSource );
			}
			catch( sax::SAXParseException &e )
			{
				rtl::OUString sLine = rtl::OUString::valueOf(e.LineNumber);
				sError = ASCII("SAXParseException occured in ") + ASCII(" Line: (") + sLine + ASCII(")");
				nRet = 3;
			}
			catch( sax::SAXException &e )
			{
				sError = ASCII("SAXException occured.");
				nRet = 4;
			}
			catch( uno::RuntimeException &e )
			{
				sError = ASCII("IOException occured.");
				nRet = 5;
			}
		
			aWriteFile.close();
			aFile.close();
		} // we MUST leave File scope
		
		
		if (nRet == 0)
		{
			FileHelper::createBackupRemoveAndRename(_aFromURL, _aToURL);
		}
		else
		{
			// remove the update file
			File::remove(_aToURL);
		}

		return nRet;
//-----------------------------------------------------------------------------
		}
	} // namespace updatetree
	
}// namespace configmgr

