/*************************************************************************
 *
 *  $RCSfile: MABNSInit.cxx,v $
 *
 *  $Revision: 1.11 $
 *
 *  last change: $Author: fs $ $Date: 2001/07/18 10:46:48 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): Darren Kenny
 *
 *
 ************************************************************************/


#include <MABNSInclude.hxx>
#include "nsIServiceManager.h"
#include "nsIEventQueueService.h"
#include "nsIChromeRegistry.h"

#include "nsIStringBundle.h"

#include "nsIDirectoryService.h"
#include "nsIProfile.h"
#include "nsIPref.h"

#include "nsString.h"
#include "nsEmbedAPI.h"
#include <sal/types.h>

#include <mozaddressbook/MABNSInit.hxx>

#include "../MABDebug.hxx"

#ifdef DEBUG
using namespace connectivity::mozaddressbook;
#endif /* DEBUG */

static nsIServiceManager*	sServiceManager = nsnull;
static PRBool				sRegistryInitializedFlag = PR_FALSE;
static PRUint32				sInitCounter = 0;
static sal_Bool				s_bProfilePresentAfterInitialized = sal_False;

static NS_DEFINE_CID(kProfileCID, NS_PROFILE_CID);
static NS_DEFINE_CID(kPrefCID, NS_PREF_CID);

#define HACK_AROUND_THREADING_ISSUES
#define HACK_AROUND_NONREENTRANT_INITXPCOM

#ifdef HACK_AROUND_NONREENTRANT_INITXPCOM
// XXX hack class to clean up XPCOM when this module is unloaded
class XPCOMCleanupHack
{
public:
    PRBool mCleanOnExit;

    XPCOMCleanupHack() : mCleanOnExit(PR_TRUE) 
    {
    }
    ~XPCOMCleanupHack()
    {
        if (mCleanOnExit)
        {
            if (sInitCounter > 0)
            {
		OSL_TRACE( "XPCOMCleanup : Cleaning up on Exit\n");
                sInitCounter = 1;
                MAB_NS_Term();
            }
            // XXX Global destructors and NS_ShutdownXPCOM don't seem to mix
//          NS_ShutdownXPCOM(sServiceManager);
        }
    }
};
static PRBool sXPCOMInitializedFlag = PR_FALSE;
static XPCOMCleanupHack sXPCOMCleanupHack;
#endif

extern "C" void NS_SetupRegistry();

static nsILocalFile *mozBinDirectory = nsnull;
static nsIDirectoryServiceProvider *appFileLocProvider = nsnull;

sal_Bool MAB_NS_Init(sal_Bool& aProfileExists)
{
    aProfileExists = sal_False ;
    nsresult rv;

    OSL_TRACE( "IN : MAB_NS_Init() \n" );
    // Reentrant calls to this method do nothing except increment a counter
    sInitCounter++;
    if (sInitCounter > 1) {
		OSL_TRACE( "OUT : MAB_NS_Init() : counter = %d\n", sInitCounter );
		aProfileExists = s_bProfilePresentAfterInitialized;
        return sal_True;
    }
#ifdef HACK_AROUND_NONREENTRANT_INITXPCOM
    // The first time, add another increment so that it'll be left until exit
    // for the final cleanup to happen
    sInitCounter++;
#endif // HACK_AROUND_NONREENTRANT_INITXPCOM

    // Initialise XPCOM
#ifdef HACK_AROUND_NONREENTRANT_INITXPCOM
    // Can't call NS_InitXPCom more than once or things go boom!
    if (!sXPCOMInitializedFlag)
#endif
    {
        // Initialise XPCOM
        NS_InitXPCOM2(&sServiceManager, mozBinDirectory, appFileLocProvider);
        if (!sServiceManager)
            return sal_False;
                
#ifdef HACK_AROUND_NONREENTRANT_INITXPCOM
        sXPCOMInitializedFlag = PR_TRUE;
        sXPCOMCleanupHack.mCleanOnExit = PR_TRUE;
#endif
    }

    // Register components
    if (!sRegistryInitializedFlag)
    {
        // XXX hack method
        NS_SetupRegistry();
        
        rv = nsComponentManager::AutoRegister(nsIComponentManager::NS_Startup,
                                              NULL /* default */);
        if (NS_FAILED(rv))
        {
            NS_ASSERTION(PR_FALSE, "Could not AutoRegister");
            return sal_False;
        }

        sRegistryInitializedFlag = PR_TRUE;
    }

    // Need to create a Profile Service
    nsCOMPtr< nsIProfile > theProfile = do_GetService( kProfileCID, &rv );
    //NS_WITH_SERVICE(nsIProfile, theProfile, kProfileCID, &rv);
    
    if (NS_SUCCEEDED(rv) ) {
	OSL_TRACE("Created an nsIProfile i/f\n");

        PRUnichar** existingProfiles = nsnull;
        PRInt32     existingProfileCnt = -1;
        theProfile->GetProfileCount( &existingProfileCnt );

	OSL_TRACE( "There are %d existing profiles \n", existingProfileCnt );
        if ( existingProfileCnt > 0 ) {
	    OSL_TRACE( "Using existing profile\n");
            PRUint32 nCnt = 0;
            theProfile->GetProfileList( &nCnt, &existingProfiles );
            if ( nCnt > 0 ) {
		OSL_TRACE( "Using existing profile #0 \n");
                nsCAutoString temp1; temp1.AssignWithConversion( existingProfiles[0] );
		OSL_TRACE("Profile Name: %s\n", NS_STATIC_CAST(const char*, temp1));
                theProfile->SetCurrentProfile( existingProfiles[0] );

                // Need to create a Pref Service
                nsCOMPtr< nsIPref > thePref = do_GetService( kPrefCID, &rv );
                //NS_WITH_SERVICE(nsIPref, thePref, kPrefCID, &rv);
                
                if (NS_SUCCEEDED(rv) ) {
		    OSL_TRACE("Created an nsIPref i/f\n");
                    //thePref->StartUp();
                    thePref->ReadUserPrefs( nsnull );
                    aProfileExists = sal_True ;
					s_bProfilePresentAfterInitialized = sal_True;
                }
            }
        }
        else {
	    OSL_TRACE( "No existing profiles\n");
        } 
    }


    OSL_TRACE( "OUT : MAB_NS_Init() - First Init\n" );

    return sal_True;

}

sal_Bool MAB_NS_Term()
{
    // Reentrant calls to this method do nothing except decrement a counter
    OSL_TRACE( "IN : MAB_NS_Term() \n" );
    if (sInitCounter > 1)
    {
        sInitCounter--;
	OSL_TRACE( "OUT : MAB_NS_Term() : counter = %d\n", sInitCounter );
        return sal_True;
    }
    sInitCounter = 0;


    NS_RELEASE(sServiceManager);

    // Terminate XPCOM & cleanup
#ifndef HACK_AROUND_NONREENTRANT_INITXPCOM
    NS_ShutdownXPCOM(sServiceManager);
#endif

    OSL_TRACE( "OUT : MAB_NS_Term() - Final Term\n" );
    return sal_True;
}

