/*************************************************************************
 *
 *  $RCSfile: querydesignaccess.cxx,v $
 *
 *  $Revision: 1.11 $
 *
 *  last change: $Author: fs $ $Date: 2001/05/10 12:18:54 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc..
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _DBAUI_QUERYDESIGNACCESS_HXX_
#include "querydesignaccess.hxx"
#endif
#ifndef _OSL_DIAGNOSE_H_
#include <osl/diagnose.h>
#endif
#ifndef DBACCESS_SHARED_DBUSTRINGS_HRC
#include "dbustrings.hrc"
#endif
#ifndef _COM_SUN_STAR_FRAME_XDISPATCHPROVIDER_HPP_
#include <com/sun/star/frame/XDispatchProvider.hpp>
#endif
#ifndef _COM_SUN_STAR_FRAME_FRAMESEARCHFLAG_HPP_
#include <com/sun/star/frame/FrameSearchFlag.hpp>
#endif
#ifndef _COMPHELPER_EXTRACT_HXX_
#include <comphelper/extract.hxx>
#endif


// .........................................................................
namespace dbaui
{
// .........................................................................

	using namespace ::com::sun::star::uno;
	using namespace ::com::sun::star::sdbc;
	using namespace ::com::sun::star::frame;
	using namespace ::com::sun::star::lang;
	using namespace ::com::sun::star::beans;
	using namespace ::com::sun::star::util;

	//======================================================================
	//= ODesignAccess
	//======================================================================
	ODesignAccess::ODesignAccess(const Reference< XMultiServiceFactory >& _rxORB)
		:m_xORB(_rxORB)
	{
		OSL_ENSURE(m_xORB.is(), "ODesignAccess::ODesignAccess: invalid service factory!");
	}

	//----------------------------------------------------------------------
	void ODesignAccess::create(const ::rtl::OUString& _rDatasourceName, const Reference< XConnection >& _rxConnection)
	{
		URL aDesignURL;
		Reference< XDispatch > xDispatcher = getDispatcher(aDesignURL);
		if (xDispatcher.is())
		{
			Sequence< PropertyValue > aArguments;
			fill(aArguments, _rDatasourceName, _rxConnection,::rtl::OUString());
			xDispatcher->dispatch(aDesignURL, aArguments);
		}
	}

	//----------------------------------------------------------------------
	Reference< XDispatch > ODesignAccess::getDispatcher(URL& _rURL)
	{
		Reference< XDispatch > xReturn;
		if (!m_xORB.is())
			return xReturn;

		_rURL.Complete = getDispatchURL();
		// TODO: let an URLTransformer complete this

		try
		{
			// get the desktop object
			Reference< XInterface > xDesktop = m_xORB->createInstance(SERVICE_FRAME_DESKTOP);
			Reference< XDispatchProvider> xProvider(xDesktop, UNO_QUERY);
			OSL_ENSURE(xProvider.is(), "ODesignAccess::getDispatcher: invalid desktop object!");

			sal_Int32 nSearchFlags = FrameSearchFlag::CHILDREN | FrameSearchFlag::CREATE;
			if (xProvider.is())
			{
				xReturn = xProvider->queryDispatch(_rURL, ::rtl::OUString::createFromAscii("_blank"), nSearchFlags);
				OSL_ENSURE(xReturn.is(), "ODesignAccess::getDispatcher: got no dispatcher!");
			}
		}
		catch(Exception&)
		{
			OSL_ENSURE(sal_False, "ODesignAccess::getDispatcher: caught an exception while retrieving the dispatcher!");
		}
		return xReturn;
	}

	//----------------------------------------------------------------------
	void ODesignAccess::edit(const ::rtl::OUString& _rDatasourceName, const ::rtl::OUString& _rName, const Reference< XConnection >& _rxConnection)
	{
		URL aDesignURL;
		Reference< XDispatch > xDispatcher = getDispatcher(aDesignURL);
		if (xDispatcher.is())
		{
			Sequence< PropertyValue > aArguments;
			fill(aArguments, _rDatasourceName, _rxConnection, _rName);
			xDispatcher->dispatch(aDesignURL, aArguments);
		}
	}

	//======================================================================
	//= OQueryDesignAccess
	//======================================================================
	//----------------------------------------------------------------------
	OQueryDesignAccess::OQueryDesignAccess(const Reference< XMultiServiceFactory >& _rxORB, sal_Bool _bCreateView, sal_Bool _bPreferSQLView)
		:ODesignAccess(_rxORB)
		,m_bCreateView(_bCreateView)
		,m_bPreferSQLView(_bPreferSQLView)
	{
	}

	//----------------------------------------------------------------------
	::rtl::OUString OQueryDesignAccess::getDispatchURL() const
	{
		return URL_COMPONENT_QUERYDESIGN;
	}

	//----------------------------------------------------------------------
	void OQueryDesignAccess::fill(Sequence< PropertyValue >& _rDispatchArguments, const ::rtl::OUString& _rDatasourceName,
		const Reference< XConnection >& _rxConnection, const ::rtl::OUString& _rQueryName)
	{
		sal_Bool bIncludeQueryName = 0 != _rQueryName.getLength();

		_rDispatchArguments.realloc(3 + (bIncludeQueryName ? 1 : 0) + (_rxConnection.is() ? 1 : 0));
		sal_Int32 nPos = 0;
		_rDispatchArguments[nPos  ].Name = PROPERTY_DATASOURCENAME;
		_rDispatchArguments[nPos++].Value <<= _rDatasourceName;

		_rDispatchArguments[nPos  ].Name = PROPERTY_QUERYDESIGNVIEW;
		_rDispatchArguments[nPos++].Value <<= ::cppu::bool2any(!m_bPreferSQLView);

		_rDispatchArguments[nPos  ].Name = PROPERTY_CREATEVIEW;
		_rDispatchArguments[nPos++].Value <<= ::cppu::bool2any(m_bCreateView);

		if (_rxConnection.is())
		{
			_rDispatchArguments[nPos  ].Name = PROPERTY_ACTIVECONNECTION;
			_rDispatchArguments[nPos++].Value <<= _rxConnection;
		}

		if (bIncludeQueryName)
		{
			_rDispatchArguments[nPos  ].Name = PROPERTY_CURRENTQUERY;
			_rDispatchArguments[nPos++].Value <<= _rQueryName;
		}
	}

	//======================================================================
	//= OTableDesignAccess
	//======================================================================
	//----------------------------------------------------------------------
	OTableDesignAccess::OTableDesignAccess(const Reference< XMultiServiceFactory >& _rxORB)	: ODesignAccess(_rxORB)
	{
	}

	//----------------------------------------------------------------------
	::rtl::OUString OTableDesignAccess::getDispatchURL() const
	{
		return URL_COMPONENT_TABLEDESIGN;
	}

	//----------------------------------------------------------------------
	void OTableDesignAccess::fill(Sequence< PropertyValue >& _rDispatchArguments, const ::rtl::OUString& _rDatasourceName,
		const Reference< XConnection >& _rxConnection, const ::rtl::OUString& _rTableName)
	{
		sal_Bool bIncludeName = 0 != _rTableName.getLength();

		_rDispatchArguments.realloc(1 + (bIncludeName ? 1 : 0) + (_rxConnection.is() ? 1 : 0));

		sal_Int32 nPos = 0;
		_rDispatchArguments[nPos  ].Name = PROPERTY_DATASOURCENAME;
		_rDispatchArguments[nPos++].Value <<= _rDatasourceName;

		if (_rxConnection.is())
		{
			_rDispatchArguments[nPos  ].Name = PROPERTY_ACTIVECONNECTION;
			_rDispatchArguments[nPos++].Value <<= _rxConnection;
		}

		if (bIncludeName)
		{
			_rDispatchArguments[nPos  ].Name = PROPERTY_CURRENTTABLE;
			_rDispatchArguments[nPos++].Value <<= _rTableName;
		}
	}
	//======================================================================
	//= ORelationDesignAccess
	//======================================================================
	//----------------------------------------------------------------------
	ORelationDesignAccess::ORelationDesignAccess(const Reference< XMultiServiceFactory >& _rxORB)	: ODesignAccess(_rxORB)
	{
	}

	//----------------------------------------------------------------------
	::rtl::OUString ORelationDesignAccess::getDispatchURL() const
	{
		return URL_COMPONENT_RELATIONDESIGN;
	}

	//----------------------------------------------------------------------
	void ORelationDesignAccess::fill(Sequence< PropertyValue >& _rDispatchArguments, const ::rtl::OUString& _rDatasourceName,
		const Reference< XConnection >& _rxConnection, const ::rtl::OUString& _rTableName)
	{

		_rDispatchArguments.realloc(1 + (_rxConnection.is() ? 1 : 0));

		sal_Int32 nPos = 0;
		_rDispatchArguments[nPos  ].Name = PROPERTY_DATASOURCENAME;
		_rDispatchArguments[nPos++].Value <<= _rDatasourceName;

		if (_rxConnection.is())
		{
			_rDispatchArguments[nPos  ].Name = PROPERTY_ACTIVECONNECTION;
			_rDispatchArguments[nPos++].Value <<= _rxConnection;
		}
	}

// .........................................................................
}	// namespace dbaui
// .........................................................................

/*************************************************************************
 * history:
 *	$Log: querydesignaccess.cxx,v $
 *	Revision 1.11  2001/05/10 12:18:54  fs
 *	OQueryDesignAccess: allow to open/create as SQL view
 *	
 *	Revision 1.10  2001/04/26 12:07:47  fs
 *	some consolidations
 *	
 *	Revision 1.9  2001/04/24 14:37:41  oj
 *	new member for views
 *	
 *	Revision 1.8  2001/03/15 08:23:44  fs
 *	cppuhelper/extract -> comphelper/extract
 *	
 *	Revision 1.7  2001/02/28 10:11:33  oj
 *	changes for the new relation design
 *	
 *	Revision 1.6  2001/02/14 14:34:19  oj
 *	tabledesign changes
 *	
 *	Revision 1.5  2001/02/05 15:40:59  oj
 *	forget to set value
 *	
 *	Revision 1.4  2001/02/05 09:47:02  oj
 *	new parameter for the designview
 *	
 *	Revision 1.3  2001/01/30 14:26:21  rt
 *	#65293# Spelling: it's all lowercase here (dbustrings.hrc)
 *	
 *	Revision 1.2  2001/01/26 16:16:45  fs
 *	renamed the string constant for the desktop service
 *	
 *	Revision 1.1  2001/01/26 14:16:57  fs
 *	initial checkin - small helper for opening a query design
 *	
 *
 *	Revision 1.0 26.01.01 11:18:06  fs
 ************************************************************************/

