/*************************************************************************
 *
 *  $RCSfile: ostatusindicatorfactory.hxx,v $
 *
 *  $Revision: 1.4 $
 *
 *  last change: $Author: as $ $Date: 2001/06/11 10:17:34 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRUNTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRUNTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc..
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef __FRAMEWORK_HELPER_OSTATUSINDICATORFACTORY_HXX_
#define __FRAMEWORK_HELPER_OSTATUSINDICATORFACTORY_HXX_

//_________________________________________________________________________________________________________________
//	my own includes
//_________________________________________________________________________________________________________________

#ifndef __FRAMEWORK_MACROS_GENERIC_HXX_
#include <macros/generic.hxx>
#endif

#ifndef __FRAMEWORK_MACROS_XINTERFACE_HXX_
#include <macros/xinterface.hxx>
#endif

#ifndef __FRAMEWORK_MACROS_DEBUG_HXX_
#include <macros/debug.hxx>
#endif

#ifndef __FRAMEWORK_THREADHELP_THREADHELPBASE_HXX_
#include <threadhelp/threadhelpbase.hxx>
#endif

#ifndef __FRAMEWORK_GENERAL_H_
#include <general.h>
#endif

//_________________________________________________________________________________________________________________
//	interface includes
//_________________________________________________________________________________________________________________

#ifndef _COM_SUN_STAR_LANG_XEVENTLISTENER_HPP_
#include <com/sun/star/lang/XEventListener.hpp>
#endif

#ifndef _COM_SUN_STAR_TASK_XSTATUSINDICATORFACTORY_HPP_
#include <com/sun/star/task/XStatusIndicatorFactory.hpp>
#endif

#ifndef _COM_SUN_STAR_TASK_XSTATUSINDICATOR_HPP_
#include <com/sun/star/task/XStatusIndicator.hpp>
#endif

#ifndef _COM_SUN_STAR_AWT_XWINDOW_HPP_
#include <com/sun/star/awt/XWindow.hpp>
#endif

#ifndef _COM_SUN_STAR_AWT_XWINDOWLISTENER_HPP_
#include <com/sun/star/awt/XWindowListener.hpp>
#endif

#ifndef _COM_SUN_STAR_LANG_EVENTOBJECT_HPP_
#include <com/sun/star/lang/EventObject.hpp>
#endif

#ifndef _COM_SUN_STAR_AWT_WINDOWEVENT_HPP_
#include <com/sun/star/awt/WindowEvent.hpp>
#endif

#ifndef _COM_SUN_STAR_LANG_XMULTISERVICEFACTORY_HPP_
#include <com/sun/star/lang/XMultiServiceFactory.hpp>
#endif

//_________________________________________________________________________________________________________________
//	other includes
//_________________________________________________________________________________________________________________

#ifndef _CPPUHELPER_WEAK_HXX_
#include <cppuhelper/weak.hxx>
#endif

#ifndef __SGI_STL_VECTOR
#include <vector>
#endif

//_________________________________________________________________________________________________________________
//	namespace
//_________________________________________________________________________________________________________________

namespace framework{

//_________________________________________________________________________________________________________________
//	exported const
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	exported definitions
//_________________________________________________________________________________________________________________

typedef ::std::vector< css::uno::Reference< css::task::XStatusIndicator > > IMPL_TIndicatorStack;

/*-************************************************************************************************************//**
	@short          implement a factory to create new status indicators
	@descr			We use it as helper for our frame implementation.
					The factory create different indicators and control his access to shared output device!
					Only the last activated component can write his state to this device.

	@implements		XInterface
					XStatusIndicatorFactory
					XWindowListener

	@base			ThreadHelpBase
					OWeakObject

	@devstatus		ready to use
*//*-*************************************************************************************************************/

class OStatusIndicatorFactory	:	public css::task::XStatusIndicatorFactory	,
									public css::awt::XWindowListener			,
									public ThreadHelpBase							,
									public ::cppu::OWeakObject
{
	//-------------------------------------------------------------------------------------------------------------
	//	public methods
	//-------------------------------------------------------------------------------------------------------------

	public:

		//---------------------------------------------------------------------------------------------------------
		//	constructor / destructor
		//---------------------------------------------------------------------------------------------------------

		/*-****************************************************************************************************//**
			@short		standard ctor
			@descr		These initialize a new instance of this class with all needed informations for work.

			@seealso	-

			@param		-
			@return		-

			@onerror	-
		*//*-*****************************************************************************************************/

	 	OStatusIndicatorFactory(	const	css::uno::Reference< css::lang::XMultiServiceFactory >&	xFactory		,
									const	css::uno::Reference< css::awt::XWindow >&				xParentWindow	);

		//---------------------------------------------------------------------------------------------------------
		//	XInterface
		//---------------------------------------------------------------------------------------------------------

		DECLARE_XINTERFACE

		//---------------------------------------------------------------------------------------------------------
		//	XStatusIndicatorFactory
		//---------------------------------------------------------------------------------------------------------

		/*-****************************************************************************************************//**
			@short		-
			@descr		-

			@seealso	-

			@param		-
			@return		-

			@onerror	-
		*//*-*****************************************************************************************************/

	    virtual css::uno::Reference< css::task::XStatusIndicator > SAL_CALL createStatusIndicator() throw( css::uno::RuntimeException );

		//---------------------------------------------------------------------------------------------------------
		//	XWindowListener
		//---------------------------------------------------------------------------------------------------------

		/*-****************************************************************************************************//**
			@short		-
			@descr		-

			@seealso	-

			@param		-
			@return		-

			@onerror	-
		*//*-*****************************************************************************************************/

    	virtual void SAL_CALL windowResized	(	const	css::awt::WindowEvent&	aEvent	) throw( css::uno::RuntimeException );
    	virtual void SAL_CALL windowMoved	(	const	css::awt::WindowEvent&	aEvent	) throw( css::uno::RuntimeException );
    	virtual void SAL_CALL windowShown	(	const	css::lang::EventObject&	aEvent	) throw( css::uno::RuntimeException );
    	virtual void SAL_CALL windowHidden	(	const	css::lang::EventObject&	aEvent	) throw( css::uno::RuntimeException );

		//---------------------------------------------------------------------------------------------------------
		//	XEventListener
		//---------------------------------------------------------------------------------------------------------

		/*-****************************************************************************************************//**
			@short		-
			@descr		-

			@seealso	-

			@param		-
			@return		-

			@onerror	-
		*//*-*****************************************************************************************************/

		virtual void SAL_CALL disposing( const css::lang::EventObject& aEvent ) throw( css::uno::RuntimeException );

		//---------------------------------------------------------------------------------------------------------
		//	public shared method!
		//---------------------------------------------------------------------------------------------------------

		/*-****************************************************************************************************//**
			@short		map XStatusIndicator acces to shared indicator implementation
			@descr		All created indicator objects of this factory will show his status messages
						in the same window! We use a special control for these but the last created indicator
						can show his information only. Thats the reason for the additional parameter "xChildIndicator"
						at follow methods. We must compare it with our current active one "xActiveIndicator".

			@seealso	service "stardiv.UnoControls.StatusIndicator"
			@seealso	interface XStatusIndicator

			@param		"xChildIndicator" reference to indicator, which will show his information
			@param		"nRange" maximum value which can be set
			@param		"nValue" new status value (must between 0 and nRange)
			@param		"sText" status text to show
			@return		-

			@onerror	Nothing will be done.
		*//*-*****************************************************************************************************/

		void start		(	const	css::uno::Reference< css::task::XStatusIndicator >&	xChildIndicator	,
							const	::rtl::OUString&									sText			,
									sal_Int32											nRange			);
		void end		(	const	css::uno::Reference< css::task::XStatusIndicator >&	xChildIndicator	);
		void reset		(	const	css::uno::Reference< css::task::XStatusIndicator >&	xChildIndicator	);
		void setText	(	const	css::uno::Reference< css::task::XStatusIndicator >&	xChildIndicator	,
							const	::rtl::OUString&									sText			);
		void setValue	(	const	css::uno::Reference< css::task::XStatusIndicator >&	xChildIndicator	,
									sal_Int32											nValue			);

	//-------------------------------------------------------------------------------------------------------------
	//	protected methods
	//-------------------------------------------------------------------------------------------------------------

	protected:

		/*-****************************************************************************************************//**
			@short		standard destructor
			@descr		This method destruct an instance of this class and clear some member.
						We protect this constructor, because you should use these instance as private and dynamicly member!

			@seealso	-

			@param		-
			@return		-

			@onerror	-
		*//*-*****************************************************************************************************/

		virtual	~OStatusIndicatorFactory();

	//-------------------------------------------------------------------------------------------------------------
	//	private methods
	//-------------------------------------------------------------------------------------------------------------

	private:

		/*-****************************************************************************************************//**
			@short		recalculate the layout
			@descr		This is neccessary if the parent windoow was changed or resized.
						We must look for that and react.

			@seealso	-

			@param		-
			@return		-

			@onerror	-
		*//*-*****************************************************************************************************/

		void impl_recalcLayout();

	//-------------------------------------------------------------------------------------------------------------
	//	debug methods
	//	(should be private everyway!)
	//-------------------------------------------------------------------------------------------------------------

		/*-****************************************************************************************************//**
			@short		debug-method to check incoming parameter of some other mehods of this class
			@descr		The following methods are used to check parameters for other methods
						of this class. The return value is used directly for an ASSERT(...).

			@seealso	ASSERTs in implementation!

			@param		references to checking variables
			@return		sal_False ,on invalid parameter
			@return		sal_True  ,otherwise

			@onerror	-
		*//*-*****************************************************************************************************/

	#ifdef ENABLE_ASSERTIONS

	private:

	 	static sal_Bool impldbg_checkParameter_OStatusIndicatorFactory	(	const	css::uno::Reference< css::lang::XMultiServiceFactory >&	xFactory		,
																			const	css::uno::Reference< css::awt::XWindow >&				xParentWindow	);
		static sal_Bool impldbg_checkParameter_windowResized			(	const	css::awt::WindowEvent&									aEvent			);
    	static sal_Bool impldbg_checkParameter_windowMoved				(	const	css::awt::WindowEvent&									aEvent			);
    	static sal_Bool impldbg_checkParameter_windowShown				(	const	css::lang::EventObject&									aEvent			);
    	static sal_Bool impldbg_checkParameter_windowHidden				(	const	css::lang::EventObject&									aEvent			);
		static sal_Bool impldbg_checkParameter_disposing				(	const	css::lang::EventObject&									aEvent			);
		static sal_Bool impldbg_checkParameter_start					(	const	css::uno::Reference< css::task::XStatusIndicator >&		xChildIndicator	,
																			const	::rtl::OUString&										sText			,
																					sal_Int32												nRange			);
		static sal_Bool impldbg_checkParameter_end						(	const	css::uno::Reference< css::task::XStatusIndicator >&		xChildIndicator	);
		static sal_Bool impldbg_checkParameter_reset					(	const	css::uno::Reference< css::task::XStatusIndicator >&		xChildIndicator	);
		static sal_Bool impldbg_checkParameter_setText					(	const	css::uno::Reference< css::task::XStatusIndicator >&		xChildIndicator	,
																			const	::rtl::OUString&										sText			);
		static sal_Bool impldbg_checkParameter_setValue					(	const	css::uno::Reference< css::task::XStatusIndicator >&		xChildIndicator	,
																					sal_Int32												nValue			);

	#endif	// #ifdef ENABLE_ASSERTIONS

	//-------------------------------------------------------------------------------------------------------------
	//	variables
	//	(should be private everyway!)
	//-------------------------------------------------------------------------------------------------------------

	private:

		IMPL_TIndicatorStack									m_aStack					;
		css::uno::Reference< css::lang::XMultiServiceFactory >	m_xFactory					;
		css::uno::Reference< css::task::XStatusIndicator >		m_xActiveIndicator			;
		css::uno::Reference< css::task::XStatusIndicator >		m_xSharedStatusIndicator	;
		css::uno::Reference< css::awt::XWindow >				m_xParentWindow				;
		css::uno::Reference< css::awt::XWindow >				m_xIndicatorWindow			;

};		//	class OStatusIndicatorFactory

}		//	namespace framework

#endif	//	#ifndef __FRAMEWORK_HELPER_OSTATUSINDICATORFACTORY_HXX_
