/*************************************************************************
 *
 *  $RCSfile: soundhandler.cxx,v $
 *
 *  $Revision: 1.3 $
 *
 *  last change: $Author: as $ $Date: 2001/07/31 06:57:28 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

//_________________________________________________________________________________________________________________
//	my own includes
//_________________________________________________________________________________________________________________

#ifndef __FRAMEWORK_DISPATCH_SOUNDHANDLER_HXX_
#include <dispatch/soundhandler.hxx>
#endif

#ifndef __FRAMEWORK_CLASSES_ARGUMENTANALYZER_HXX_
#include <classes/argumentanalyzer.hxx>
#endif

#ifndef __FRAMEWORK_THREADHELP_TRANSACTIONGUARD_HXX_
#include <threadhelp/transactionguard.hxx>
#endif

#ifndef __FRAMEWORK_THREADHELP_WRITEGUARD_HXX_
#include <threadhelp/writeguard.hxx>
#endif

#ifndef __FRAMEWORK_THREADHELP_READGUARD_HXX_
#include <threadhelp/readguard.hxx>
#endif

#ifndef __FRAMEWORK_MACROS_DEBUG_HXX_
#include <macros/debug.hxx>
#endif

#ifndef __FRAMEWORK_SERVICES_H_
#include <services.h>
#endif

//_________________________________________________________________________________________________________________
//	interface includes
//_________________________________________________________________________________________________________________

#ifndef _COM_SUN_STAR_IO_XINPUTSTREAM_HPP_
#include <com/sun/star/io/XInputStream.hpp>
#endif

//_________________________________________________________________________________________________________________
//	includes of other projects
//_________________________________________________________________________________________________________________

#ifndef _RTL_USTRBUF_HXX_
#include <rtl/ustrbuf.hxx>
#endif

#ifndef _SV_SALSTYPE_HXX
#include <vcl/salstype.hxx>
#endif

//_________________________________________________________________________________________________________________
//	namespace
//_________________________________________________________________________________________________________________

namespace framework{

//_________________________________________________________________________________________________________________
//	non exported const
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	non exported definitions
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	declarations
//_________________________________________________________________________________________________________________

//*****************************************************************************************************************
//  XInterface, XTypeProvider, XServiceInfo
//*****************************************************************************************************************
DEFINE_XINTERFACE_5                 (   SoundHandler                                                 ,
                                        OWeakObject                                                  ,
                                        DIRECT_INTERFACE( css::lang::XTypeProvider                  ),
                                        DIRECT_INTERFACE( css::lang::XServiceInfo                   ),
                                        DIRECT_INTERFACE( css::lang::XComponent                     ),
                                        DIRECT_INTERFACE( css::frame::XDispatch                     ),
                                        DIRECT_INTERFACE( css::document::XExtendedFilterDetection   )
                                    )

DEFINE_XTYPEPROVIDER_5              (   SoundHandler                                                ,
                                        css::lang::XTypeProvider                                    ,
                                        css::lang::XServiceInfo                                     ,
                                        css::lang::XComponent                                       ,
                                        css::frame::XDispatch                                       ,
                                        css::document::XExtendedFilterDetection
                                    )

DEFINE_XSERVICEINFO_MULTISERVICE    (   SoundHandler                                                ,
                                        ::cppu::OWeakObject                                         ,
                                        SERVICENAME_SOUNDHANDLER                                    ,
                                        IMPLEMENTATIONNAME_SOUNDHANDLER
									)

DEFINE_INIT_SERVICE                 (   SoundHandler,
                                        {
                                        }
                                    )

/*-************************************************************************************************************//**
    @short      standard ctor
    @descr      These initialize a new instance of ths class with needed informations for work.

    @seealso    using at owner

    @param      "xFactory", reference to service manager for creation of new services
    @return     -

    @onerror    Show an assertion and do nothing else.
    @threadsafe yes
*//*-*************************************************************************************************************/
SoundHandler::SoundHandler( const css::uno::Reference< css::lang::XMultiServiceFactory >& xFactory )
		//	Init baseclasses first
        :   ThreadHelpBase      (                                   )
        ,   TransactionBase     (                                   )
        ,   ::cppu::OWeakObject (                                   )
        // Init member
        ,   m_xFactory          ( xFactory                          )
        ,   m_aStatusListener   ( m_aLock.getShareableOslMutex()    )
        ,   m_aEventListener    ( m_aLock.getShareableOslMutex()    )
        ,   m_aPlayer           (                                   )
{
	// Safe impossible cases
    LOG_ASSERT2( impldbg_checkParameter_SoundHandler( xFactory ), "SoundHandler::SoundHandler()", "Invalid parameter detected!" )

    // Start listening for player notifications.
    // Do it one time only. We need this notifications every time ... and its faster
    // then registering during every dispatch() call!
    m_aPlayer.SetNotifyHdl( LINK( this, SoundHandler, implts_PlayerNotify ) );

    // Enable object for real working.
    m_aTransactionManager.setWorkingMode( E_WORK );
}

/*-************************************************************************************************************//**
    @short      standard dtor
    @descr      Normaly we are destroyed by caling dispose(). Bute sometimes we must die by reaching a refcount of 0.
                It's not fine ... but what should we do.
                If we detect these state (normaly dispose() disable object for further working by setting internal mode to E_CLOSE!)
                we should dipose us.

    @seealso    method dispose()

    @param      -
    @return     -

    @onerror    -
    @threadsafe yes
*//*-*************************************************************************************************************/
SoundHandler::~SoundHandler()
{
    if( m_aTransactionManager.getWorkingMode()!=E_CLOSE )
    {
        dispose();
    }
}

/*-************************************************************************************************************//**
    @interface  ::com::sun::star::frame::XDispatch

    @short      try to load audio file
    @descr      This method try to load given audio file by URL and play it. We use vcl/Sound class to do that.
                If you whish to get informations about loading state of given URL, you should be a status listener
                on this instance too.

    @attention  Look for right using of any mutex or rw-locks here ... because we can called back by used player instance
                in "implts_PlayerNotify()". So you should synchronize it ...
                Normaly we don't need it here ...
                We have registered thsi operation as non breakable transaction. Our player instance live, if we live.
                Our dispose() call wait for finishing of our transaction ... So we can work without any interuption here.
                I hope it :-)

    @seealso    class vcl/Sound

    @param      "aURL"      , URL to dispatch.
    @param      "lArguments", list of optional arguments.
    @return     -

    @onerror    -
    @threadsafe yes
*//*-*************************************************************************************************************/
void SAL_CALL SoundHandler::dispatch( const css::util::URL&                                  aURL       ,
                                      const css::uno::Sequence< css::beans::PropertyValue >& lArguments ) throw( css::uno::RuntimeException )
{
    /* UNSAFE AREA --------------------------------------------------------------------------------------------- */
    // Safe impossible cases ... check for wrong call parameters
    LOG_ASSERT2( impldbg_checkParameter_dispatch( aURL, lArguments ), "SoundHandler::dispatch()", "Invalid parameter detected." )
    // Use transaction manager to reject wrong calls only!
    // Stop transaction immediatly - because: We are listener for sound notifications. (see ctor)
    // If we call one of follow methods on our player - sometimes he call us back. In these callback
    // we send status events to our listener. These listener release his references ... and may be WE WILL DIE - called by dispose or our dtor :-(
    // But our dispose() method wait for already registered transactions.
    // That's why we must deregister it before we start these possible cycle.

    // => By the way: Don't use a dynamical member as player! He must live if these instance live!
    // Otherwhise he could be deleted during this method ...
    TransactionGuard aTransaction( m_aTransactionManager, E_HARDEXCEPTIONS );
    aTransaction.stop();

    // Stop playing of last dispatched sound ... before you start a new one.
    m_aPlayer.Stop();

    // Set file name ... and try to start playing again.
    // It will be an asynchronous operation.
    // Stop it in dispose() ... or in implts_PlayerNotify() ... or at new dispatch() calls (seee before!)
    if( m_aPlayer.SetSoundName( aURL.Complete ) == sal_True )
    {
        m_aPlayer.Play();
    }
}

/*-************************************************************************************************************//**
    @interface  ::com::sun::star::frame::XDispatch

    @short      add/remove listener for state events
    @descr      You can add/remove a listener to get information about status of dispatch calls.
                But we return no further informations ... "ok" or "fail" only!

    @seealso    member m_aListenerContainer

    @param      "xListener" , reference to a valid listener for state events.
    @param      "aURL"      , URL about listener will be informed, if something occured.
    @return     -

    @onerror    We do nothing.
    @threadsafe yes
*//*-*************************************************************************************************************/
void SAL_CALL SoundHandler::addStatusListener( const css::uno::Reference< css::frame::XStatusListener >& xListener ,
                                               const css::util::URL&                                     aURL      ) throw( css::uno::RuntimeException )
{
    /* UNSAFE AREA --------------------------------------------------------------------------------------------- */
    // Safe impossible cases ... check for wrong call parameters
    LOG_ASSERT2( impldbg_checkParameter_addStatusListener( xListener, aURL ), "SoundHandler::addStatusListener()", "Invalid parameter detected." )
    // Register method as transaction or reject wrong calls!
    TransactionGuard aTransaction( m_aTransactionManager, E_HARDEXCEPTIONS );

    // Add listener to container ...
    // We don't need any mutex here .. because our baseclass is threadsafe by himself!
    // And this member live, if we live (till transaction manager reject wrong calls).
    m_aStatusListener.addInterface( aURL.Complete, xListener );
}

//*****************************************************************************************************************
void SAL_CALL SoundHandler::removeStatusListener( const css::uno::Reference< css::frame::XStatusListener >& xListener ,
                                                  const css::util::URL&                                     aURL      ) throw( css::uno::RuntimeException )
{
    /* UNSAFE AREA --------------------------------------------------------------------------------------------- */
    // Safe impossible cases ... check for wrong call parameters
    LOG_ASSERT2( impldbg_checkParameter_removeStatusListener( xListener, aURL ), "SoundHandler::removeStatusListener()", "Invalid parameter detected." )
    // Register method as transaction or reject wrong calls!
    // Attention!: We must allow deregistration of listener during our own dispose() call ...
    // So we have to throw exceptions for working mode E_CLOSED only - not for E_BEFOREDISPOSE! => E_SOFTEXCEPTIONS
    TransactionGuard aTransaction( m_aTransactionManager, E_SOFTEXCEPTIONS );

    // Remove listener to container ...
    // We don't need any mutex here .. because our baseclass is threadsafe by himself!
    // And this member live, if we live (till transaction manager reject wrong calls).
    m_aStatusListener.removeInterface( aURL.Complete, xListener );
}

/*-************************************************************************************************************//**
    @interface  ::com::sun::star::document::XExtendedFilterDetection

    @short      try to detect file (given as argument included in "lDescriptor")
    @descr      We try to detect, if given file could be handled by this class and is a well known one.
                If it is - we return right internal type name - otherwise we return nothing!
                So call can search for another detect service and ask him too.

    @attention  a) We don't need any mutex here ... because we don't use any member!
                b) Dont' use internal player instance "m_aPlayer" to detect given sound file!
                   It's not neccessary to do that ... and we can use temp. variable to do the same.
                   This way is easy - we don't must synchronize it with currently played sounds!
                   Another reason to do so ... We are a listener on our internal ma_Player object.
                   If you would call "IsSoundFile()" on this instance, he would call us back and we start
                   sending of a status event to our listener. But this mechanism is reserved
                   for dispatch()/statusChanged()!

    @seealso    -

    @param      "lDescriptor", description of file to detect
    @return     Internal type name which match this file ... or nothing if it is unknown.

    @onerror    We return nothing.
    @threadsafe yes
*//*-*************************************************************************************************************/
::rtl::OUString SAL_CALL SoundHandler::detect( css::uno::Sequence< css::beans::PropertyValue >& lDescriptor ) throw( css::uno::RuntimeException )
{
    /* UNSAFE AREA --------------------------------------------------------------------------------------------- */
    // Safe impossible cases ... check for wrong call parameters
    LOG_ASSERT2( impldbg_checkParameter_detect( lDescriptor ), "SoundHandler::detect()", "Invalid parameter detected." )
    // Register method as transaction or reject wrong calls!
    TransactionGuard aTransaction( m_aTransactionManager, E_HARDEXCEPTIONS );

    // Our default is "nothing". So we can return it, if detection failed or fily type is realy unknown.
    ::rtl::OUString sTypeName;

    // Analyze given descriptor to find filename or input stream or ...
    ArgumentAnalyzer                                aAnalyzer( lDescriptor );
    ::rtl::OUString                                 sURL                    ;
    css::uno::Reference< css::io::XInputStream >    xStream                 ;
    sal_Bool                                        bURLExist               ;
    sal_Bool                                        bStreamExist            ;
    Sound                                           aDetector               ;

    bURLExist    = aAnalyzer.getArgument( E_URL        , sURL    );
    bStreamExist = aAnalyzer.getArgument( E_INPUTSTREAM, xStream );

    // A stream should be used if her exist .. otherwise we must use URL.
    // So we avoid against multiple openining of file!
    if( bStreamExist == sal_True )
    {
        LOG_WARNING( "SoundHandler::detect()", "Detection by using stream not implemented yet!" )
    }
    else
    if( bURLExist == sal_True )
    {
        if( aDetector.IsSoundFile( sURL ) == sal_True )
        {
            // "IsSoundFile" idffer between different "wav" and "au" file versions ...
            // couldn't return this information ... because: He use the OS to detect it!
            // I think we can the following ones:
            //  a) look for given extension of url to map our type decision HARD CODED!!!
            //  b) return preferred type every time... it's easy :-)
            sTypeName = DECLARE_ASCII("wav_Wave_Audio_File");
            aAnalyzer.setArgument( E_TYPENAME, sTypeName );
        }
    }

    // Return our decision.
    return sTypeName;
}

/*-****************************************************************************************************//**
    @interface  ::com::sun::star::lang::XComponent

	@short		destroy instance
	@descr		The owner of this object calles the dispose method if the object
				should be destroyed. All other objects and components, that are registered
				as an EventListener are forced to release their references to this object.

    @seealso    interface XComponent

	@param		-
	@return		-

    @onerror    -
    @threadsafe yes
*//*-*****************************************************************************************************/
void SAL_CALL SoundHandler::dispose() throw( css::uno::RuntimeException )
{
	/*ATTENTION
		Make it threadsafe ... but this method is a special one!
		We must close objet for working BEFORE we dispose it realy ...
        After successful closing all interface calls are rejected by our
        transaction manager automaticly.
	 */

    /* SAFE AREA ----------------------------------------------------------------------------------------------- */
    WriteGuard aWriteLock( m_aLock );

    // Look for multiple calls of this method!
    // Use write lock to have an exclusiv access ... because
    // otherwise more then one caller could pass this gate ...
    TransactionGuard aTransaction( m_aTransactionManager, E_HARDEXCEPTIONS );

    // We are alone and its the first call of this method ...
    // otherwise call before must throw a DisposedException!
    // Don't forget to release this registered transaction here ...
    // because next "setWorkingMode()" call blocks till all current existing one
    // are finished!
    aTransaction.stop();

    // Disable this instance for further work.
    // This will wait for all current running ones ...
    // and reject all further requests!
    m_aTransactionManager.setWorkingMode( E_BEFORECLOSE );

    // We should hold a reference to ourself ...
    // because our owner dispose us and release our reference ...
    // May be we will die before we could finish this method ...
    css::uno::Reference< css::frame::XFrame > xThis( static_cast< ::cppu::OWeakObject* >(this), css::uno::UNO_QUERY );

    // Now we can release our lock!
    // We should be alone and further dispose calls are rejected by lines before ...
    // I hope it :-)
    aWriteLock.unlock();
    /* UNSAFE AREA --------------------------------------------------------------------------------------------- */

    // Stop playing .. if any player is still active!
    // Status events are sended by callback "implts_PlayerNotify()" automaticly.
    // Attention: These caller of this dispose() method must be already deregistered at this time!!!
    // Otherwise it's a bug. It's nott allowed to be a listener on objects and dispose it himself.
    // A never ending call stack will occure then: dispose - send event - dispose - send event ... and so on.
    // But if other listener for this event exist - we must notify them.
    m_aPlayer.Stop();

    // Send message to all listener and forget her references.
    // They call us back in "removeStatusListener()" only.
    // These methods can work by using E_SOFTEXCEPTIONS! (see there)
    // But all other methods are not allowed!!
    LOG_DISPOSEEVENT( "SoundHandler", "SoundHandler" )
    css::lang::EventObject aEvent( xThis );
    m_aStatusListener.disposeAndClear( aEvent );
    m_aEventListener.disposeAndClear ( aEvent );

    // Forget all other references ...
    m_xFactory = css::uno::Reference< css::lang::XMultiServiceFactory >();

    // Disable this instance for further working realy!
    m_aTransactionManager.setWorkingMode( E_CLOSE );
}

/*-************************************************************************************************************//**
    @interface  ::com::sun::star::lang::XComponent

    @short      add/remove listener for disposing events
    @descr      If I should die ... you must forghet me :-(
                So I send a message to you and you must release all references to this object.

    @seealso    interface XComponent
    @seealso    member m_aListenerContainer

    @param      "xListener" , reference to a valid listener for disposing events.
    @return     -

    @onerror    We do nothing.
    @threadsafe yes
*//*-*************************************************************************************************************/
void SAL_CALL SoundHandler::addEventListener( const css::uno::Reference< css::lang::XEventListener >& xListener ) throw( css::uno::RuntimeException )
{
    // Safe impossible cases ... check for wrong call parameters
    LOG_ASSERT2( impldbg_checkParameter_addEventListener( xListener ), "SoundHandler::addEventListener()", "Invalid parameter detected." )
    // Register method as transaction or reject wrong calls!
    TransactionGuard aTransaction( m_aTransactionManager, E_HARDEXCEPTIONS );

    // Add listener to container ...
    // We don't need any mutex here .. because our baseclass is threadsafe by himself!
    // And this member live, if we live (till transaction manager reject wrong calls).
    m_aEventListener.addInterface( ::getCppuType( (const css::uno::Reference< css::lang::XEventListener >*)NULL ), xListener );
}

//*****************************************************************************************************************
void SAL_CALL SoundHandler::removeEventListener( const css::uno::Reference< css::lang::XEventListener >& xListener ) throw( css::uno::RuntimeException )
{
    // Safe impossible cases ... check for wrong call parameters
    LOG_ASSERT2( impldbg_checkParameter_removeEventListener( xListener ), "SoundHandler::removeEventListener()", "Invalid parameter detected." )
    // Register method as transaction or reject wrong calls!
    // Attention!: We must allow deregistration of listener during our own dispose() call ...
    // So we have to throw exceptions for working mode E_CLOSED only - not for E_BEFOREDISPOSE! => E_SOFTEXCEPTIONS
    TransactionGuard aTransaction( m_aTransactionManager, E_SOFTEXCEPTIONS );

    // Remove listener to container ...
    // We don't need any mutex here .. because our baseclass is threadsafe by himself!
    // And this member live, if we live (till transaction manager reject wrong calls).
    m_aEventListener.removeInterface( ::getCppuType( (const css::uno::Reference< css::lang::XEventListener >*)NULL ), xListener );
}

/*-************************************************************************************************************//**
    @short      send status event to alllistener fwhich registered for this URL.
    @descr      You can add listener to get informations about loading state of dispatched URL's.
                With these little helper method we send right event to all registered listener.

    @attention  a)
                This method is threadsafe - because we use threadsafe baseclass ... but
                transaction will not registered and we don't reject any call.
                The only way - we can be called - is from inside ... And in this case
                a transaction should be already registered!
                b)
                This code is threadsafe ... and use a pointer to an OInterfaceContainerHelper.
                These helper share mutex with our container and react for problems during our iteration about it!

    @seealso    member m_aListenerContainer

    @param      "sURL"      , loaded URL
    @param      "bLoadState", state of load operation => will be state of event
    @return     -

    @onerror    We do nothing.
    @threadsafe yes
*//*-*************************************************************************************************************/
void SoundHandler::implts_sendStatusEvent( const ::rtl::OUString& sURL       ,
                                                 sal_Bool         bLoadState )
{
	// Get listener for given URL!
    ::cppu::OInterfaceContainerHelper* pListenerForURL = m_aStatusListener.getContainer( sURL );
	// Send messages to all listener.
	// Do nothing, if there no listener or "getContainer()" works not correct!
	if( pListenerForURL != NULL )
	{
		// Build event for send to listener.
        css::frame::FeatureStateEvent aEvent;
        aEvent.FeatureURL.Complete  =   sURL                                                                                                          ;
        aEvent.FeatureDescriptor    =   FEATUREDESCRIPTOR_LOADSTATE                                                                                   ;
        aEvent.IsEnabled            =   bLoadState                                                                                                    ;
        aEvent.Requery              =   sal_False                                                                                                     ;
        aEvent.State              <<=   css::uno::Reference< css::uno::XInterface >( static_cast< ::cppu::OWeakObject* >(this), css::uno::UNO_QUERY ) ;
		// Send message to all listener on this URL.
        ::cppu::OInterfaceIteratorHelper pIterator( *pListenerForURL );
        while( pIterator.hasMoreElements() )
		{
            ((css::frame::XStatusListener*)pIterator.next())->statusChanged( aEvent );
		}
	}
}

/*-************************************************************************************************************//**
    @short      -
    @descr      -

    @seealso    -

    @param      -
    @return     -

    @onerror    -
    @threadsafe -
*//*-*************************************************************************************************************/
IMPL_LINK( SoundHandler, implts_PlayerNotify, void*, pVoid )
{
    /* UNSAFE AREA --------------------------------------------------------------------------------------------- */
    // Register method as transaction or reject wrong calls!
    // May be - this notification comes during our dispose() method ...
    // so we must use E_SOFTEXCEPTIONS here.
    TransactionGuard aTransaction( m_aTransactionManager, E_SOFTEXCEPTIONS );

    /* SAFE AREA ----------------------------------------------------------------------------------------------- */
    ReadGuard aReadLock( m_aLock );

    // Try to get internal state of our player to react.
    // nError isn't realy used yet ... but we think about new status event format ...
    SoundNotification eNotify  = m_aPlayer.GetNotification();
    sal_uInt32        nError   = m_aPlayer.GetLastError();

    // Attention: We must reset internal sound name of player ... because
    //            sound class couldn't open same file twice :-(
    //            But don't forget to save currently set file name for sending status events to our listener.
    ::rtl::OUString sSoundFile = m_aPlayer.GetSoundName();
    m_aPlayer.SetSoundName( ::rtl::OUString() );

    // Don't hold lock during sending of status events!
    aReadLock.unlock();

    // Unregister transaction here too!!!
    // Because - we send status to our owner ... and may be he forget our reference ... or call
    // dispose(). So we will block during waiting dispose() call for already registered transactions!
    // Thats the reason to deregister this one.
    aTransaction.stop();
    /* UNSAFE AREA --------------------------------------------------------------------------------------------- */

    // React for notification ...
    // and send right status to our listener.
    switch( eNotify )
    {
        case SOUND_NOTIFY_SUCCESS   :   {
                                            implts_sendStatusEvent( sSoundFile, sal_True  );
                                        }
                                        break;
        case SOUND_NOTIFY_ABORTED   :   {
                                            implts_sendStatusEvent( sSoundFile, sal_False );
                                        }
                                        break;
        case SOUND_NOTIFY_ERROR     :   {
                                            implts_sendStatusEvent( sSoundFile, sal_False );
                                        }
                                        break;
        #ifdef ENABLE_WARNINGS
        case SOUND_NOTIFY_NOTHING   :   LOG_WARNING( "SoundHandler::implts_PlayerNotify()", "Notify: \"SOUND_NOTIFY_NOTHING\"? What should I do?" )
                                        break;
        default                     :   LOG_WARNING( "SoundHandler::implts_PlayerNotify()", "Unknown SoundNotification detected! Please implement it ..." )
                                        break;
        #endif
    }

    return 0;
}

/*-************************************************************************************************************//**
    @short      debug-methods to check incoming parameter of our interface mehods
    @descr      The following methods are used to check incoming parameters
                of this class. The return value is used directly for an ASSERT(...).
                So we control rightt using of this class an guarantee correct results of operations.

    @seealso    ASSERTs in implementation!

    @param      References to checking variables
    @return     false on invalid parameter<BR>
                true  otherway

    @onerror    We return false.
    @threadsafe They mustn't be threadsafe - because they are static and work on given parameters only!
*//*-*************************************************************************************************************/
#ifdef ENABLE_ASSERTIONS

//*****************************************************************************************************************
sal_Bool SoundHandler::impldbg_checkParameter_SoundHandler( const css::uno::Reference< css::lang::XMultiServiceFactory >& xFactory )
{
    return   (
                ( &xFactory      == NULL      ) ||
                (  xFactory.is() == sal_False )
             );
}

//*****************************************************************************************************************
sal_Bool SoundHandler::impldbg_checkParameter_dispatch( const css::util::URL&                                  aURL       ,
                                                        const css::uno::Sequence< css::beans::PropertyValue >& lArguments )
{
    return(
            ( &aURL                     ==  NULL    )   ||
            ( aURL.Complete.getLength() <   1       )   ||
            ( &lArguments               ==  NULL    )
          );
}

//*****************************************************************************************************************
sal_Bool SoundHandler::impldbg_checkParameter_addStatusListener( const css::uno::Reference< css::frame::XStatusListener >& xListener ,
                                                                 const css::util::URL&                                     aURL      )
{
    return   (
                ( &xListener                ==  NULL        )   ||
                ( xListener.is()            ==  sal_False   )   ||
                ( &aURL                     ==  NULL        )   ||
                ( aURL.Complete.getLength() <   1           )
             );
}

//*****************************************************************************************************************
sal_Bool SoundHandler::impldbg_checkParameter_removeStatusListener( const css::uno::Reference< css::frame::XStatusListener >& xListener ,
                                                                    const css::util::URL&                                     aURL      )
{
    return   (
                ( &xListener                ==  NULL        )   ||
                ( xListener.is()            ==  sal_False   )   ||
                ( &aURL                     ==  NULL        )   ||
                ( aURL.Complete.getLength() <   1           )
             );
}

//*****************************************************************************************************************
sal_Bool SoundHandler::impldbg_checkParameter_addEventListener( const css::uno::Reference< css::lang::XEventListener >& xListener )
{
    return   (
                ( &xListener     ==  NULL        )   ||
                ( xListener.is() ==  sal_False   )
             );
}

//*****************************************************************************************************************
sal_Bool SoundHandler::impldbg_checkParameter_removeEventListener( const css::uno::Reference< css::lang::XEventListener >& xListener )
{
    return   (
                ( &xListener     ==  NULL        )   ||
                ( xListener.is() ==  sal_False   )
             );
}

//*****************************************************************************************************************
sal_Bool SoundHandler::impldbg_checkParameter_detect( css::uno::Sequence< css::beans::PropertyValue >& lDescriptor )
{
    return   (
                ( &lDescriptor            == NULL )    ||
                ( lDescriptor.getLength() <  1    )
             );
}

#endif	//	#ifdef ENABLE_ASSERTIONS

}		//	namespace framework
