/*************************************************************************
 *
 *  $RCSfile: ostatusindicator.cxx,v $
 *
 *  $Revision: 1.3 $
 *
 *  last change: $Author: as $ $Date: 2001/06/11 10:31:18 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRUNTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRUNTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc..
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

//_________________________________________________________________________________________________________________
//	my own includes
//_________________________________________________________________________________________________________________

#ifndef __FRAMEWORK_HELPER_OSTATUSINDICATOR_HXX_
#include <helper/ostatusindicator.hxx>
#endif

//_________________________________________________________________________________________________________________
//	interface includes
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	includes of other projects
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	namespace
//_________________________________________________________________________________________________________________

namespace framework{

using namespace ::com::sun::star::task			;
using namespace ::com::sun::star::lang			;
using namespace ::com::sun::star::uno			;
using namespace ::cppu							;
using namespace ::osl							;
using namespace ::rtl							;
using namespace ::std							;

//_________________________________________________________________________________________________________________
//	non exported const
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	non exported definitions
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	declarations
//_________________________________________________________________________________________________________________

//*****************************************************************************************************************
//	constructor
//*****************************************************************************************************************
OStatusIndicator::OStatusIndicator(	Mutex&						aMutex		,
									OStatusIndicatorFactory*	aFactory	)
		//	Init baseclasses first
		:	OWeakObject			(				)
		// Init member
		,	m_aMutex			( aMutex		)
		,	m_pIndicatorFactory	( aFactory		)
		,	m_xFactoryWeak		( aFactory		)
		,	m_sText				( OUString()	)
		,	m_nRange			( 0				)
		,	m_nValue			( 0				)
{
	// Safe impossible cases
	// Method not defined for all incoming parameter.
	LOG_ASSERT( impldbg_checkParameter_OStatusIndicator( aMutex, aFactory ), "OStatusIndicator::OStatusIndicator()\nInvalid parameter detected!\n" )
}

//*****************************************************************************************************************
//	destructor
//*****************************************************************************************************************
OStatusIndicator::~OStatusIndicator()
{
}

//*****************************************************************************************************************
//	XInterface, XTypeProvider
//*****************************************************************************************************************
DEFINE_XINTERFACE_2		(	OStatusIndicator					,
							OWeakObject					  		,
							DIRECT_INTERFACE(XTypeProvider		),
							DIRECT_INTERFACE(XStatusIndicator	)
						)

DEFINE_XTYPEPROVIDER_2	(	OStatusIndicator					,
							XTypeProvider						,
							XStatusIndicator
						)

//*****************************************************************************************************************
//	XStatusIndicator
//*****************************************************************************************************************
void SAL_CALL OStatusIndicator::start(	const	OUString&	sText	,
			  									sal_Int32	nRange	) throw( RuntimeException )
{
	// Ready for multithreading
    ::osl::MutexGuard aGuard( m_aMutex );
	// Safe impossible cases
	// Method not defined for all incoming parameter.
	LOG_ASSERT( impldbg_checkParameter_start( sText, nRange ), "OStatusIndicator::start()\nInvalid parameter detected!\n" )

	// Take given values, safe it for later activation (if we are not the most active indicator of our factory!)
	m_sText		=	sText	;
	m_nRange	=	nRange	;
	m_nValue	=	0		;
	// and forward operation to our shared indicator window.
	// Do the follow only, if weak instance is valid!
	// Lock it for follow operations - make a "hard reference"!
	// If weak valid - pointer to implementation is valid too ...!!!
	Reference< XInterface > xLock( m_xFactoryWeak.get(), UNO_QUERY );
	if( xLock.is() == sal_True )
	{
		m_pIndicatorFactory->start( this, m_sText, m_nRange );
	}
}

//*****************************************************************************************************************
//	XStatusIndicator
//*****************************************************************************************************************
void SAL_CALL OStatusIndicator::end() throw( RuntimeException )
{
	// Ready for multithreading
    ::osl::MutexGuard aGuard( m_aMutex );

	// Attention: With follow calls our factory forget us ...
	// and if his reference to this object was the last one ...
	// we will die! We must protect us againt these till this method end.
	// We hold us by himself!
	Reference< XInterface > xProtectionAgainstDie( static_cast< OWeakObject* >( this ), UNO_QUERY );

	// Reset internal values.
	m_sText		=	OUString()	;
	m_nRange	=	100			;
	m_nValue	=	0			;
	// Do the follow only, if weak instance is valid!
	// Lock it for follow operations - make a "hard reference"!
	// If weak valid - pointer to implementation is valid too ...!!!
	Reference< XInterface > xLock( m_xFactoryWeak.get(), UNO_QUERY );
	if ( xLock.is() == sal_True )
	{
		// Finish all operations on shared indicator ...
		m_pIndicatorFactory->end( this );
		// Then delete all connections to the factory and forget al other references.
		// This instance will never work from this time again!
		// From now all try to lock the weak reference will breakdown.
		m_xFactoryWeak		=	WeakReference< XStatusIndicatorFactory >();
		m_pIndicatorFactory	=	NULL;
	}
}

//*****************************************************************************************************************
//	XStatusIndicator
//*****************************************************************************************************************
void SAL_CALL OStatusIndicator::reset() throw( RuntimeException )
{
	// Ready for multithreading
    ::osl::MutexGuard aGuard( m_aMutex );

	// Reset internal values to defaults.
	m_sText		=	OUString()	;
	m_nRange	=	100			;
	m_nValue	=	0			;
	// Forward operation to shared indicator.
	// Do the follow only, if weak instance is valid!
	// Lock it for follow operations - make a "hard reference"!
	// If weak valid - pointer to implementation is valid too ...!!!
	Reference< XInterface > xLock( m_xFactoryWeak.get(), UNO_QUERY );
	if ( xLock.is() == sal_True )
	{
		m_pIndicatorFactory->reset( this );
	}
}

//*****************************************************************************************************************
//	XStatusIndicator
//*****************************************************************************************************************
void SAL_CALL OStatusIndicator::setText( const OUString& sText ) throw( RuntimeException )
{
	// Ready for multithreading
    ::osl::MutexGuard aGuard( m_aMutex );
	// Safe impossible cases
	// Method not defined for all incoming parameter.
	LOG_ASSERT( impldbg_checkParameter_setText( sText ), "OStatusIndicator::setText()\nInvalid parameter detected!\n" )

	// Safe given parameter and show it.
	// We must save it, because we can't be the latest created indicator of factory.
	// Then we must hold our values til reactivation!
	m_sText = sText;
	// Forward operation to shared indicator.
	// Do the follow only, if weak instance is valid!
	// Lock it for follow operations - make a "hard reference"!
	// If weak valid - pointer to implementation is valid too ...!!!
	Reference< XInterface > xLock( m_xFactoryWeak.get(), UNO_QUERY );
	if ( xLock.is() == sal_True )
	{
		m_pIndicatorFactory->setText( this, m_sText );
	}
}

//*****************************************************************************************************************
//	XStatusIndicator
//*****************************************************************************************************************
void SAL_CALL OStatusIndicator::setValue( sal_Int32 nValue ) throw( RuntimeException )
{
	// Ready for multithreading
    ::osl::MutexGuard aGuard( m_aMutex );
	// Safe impossible cases
	// Method not defined for all incoming parameter.
	LOG_ASSERT( impldbg_checkParameter_setValue( nValue, m_nRange ), "OStatusIndicator::setValue()\nInvalid parameter detected!\n" )

	// Safe given parameter and show it.
	// We must save it, because we can't be the latest created indicator of factory.
	// Then we must hold our values til reactivation!
	m_nValue = nValue;
	// Forward operation to shared indicator.
	// Do the follow only, if weak instance is valid!
	// Lock it for follow operations - make a "hard reference"!
	// If weak valid - pointer to implementation is valid too ...!!!
	Reference< XInterface > xLock( m_xFactoryWeak.get(), UNO_QUERY );
	if ( xLock.is() == sal_True )
	{
		m_pIndicatorFactory->setValue( this, m_nValue );
	}
}

//*****************************************************************************************************************
//	public implementation methods
//*****************************************************************************************************************
void OStatusIndicator::impl_activate()
{
	// Ready for multithreading
	// Normaly impl methods are not threadsafe ... but these will be called as public!
    ::osl::MutexGuard aGuard( m_aMutex );

	// Do the follow only, if weak instance is valid!
	// Lock it for follow operations - make a "hard reference"!
	// If weak valid - pointer to implementation is valid too ...!!!
	Reference< XInterface > xLock( m_xFactoryWeak.get(), UNO_QUERY );
	if ( xLock.is() == sal_True )
	{
		// Our factory say: "You are the most active indicator now ... show your values again!".
		// OK - we will do it.
		m_pIndicatorFactory->start		( this, m_sText	, m_nRange	);
		m_pIndicatorFactory->setValue	( this, m_nValue			);
	}
}

//_________________________________________________________________________________________________________________
//	debug methods
//_________________________________________________________________________________________________________________

/*-----------------------------------------------------------------------------------------------------------------
	The follow methods checks the parameter for other functions. If a parameter or his value is non valid,
	we return "sal_False". (else sal_True) This mechanism is used to throw an ASSERT!

	ATTENTION

		If you miss a test for one of this parameters, contact the autor or add it himself !(?)
		But ... look for right testing! See using of this methods!
-----------------------------------------------------------------------------------------------------------------*/

#ifdef ENABLE_ASSERTIONS

//*****************************************************************************************************************
sal_Bool OStatusIndicator::impldbg_checkParameter_OStatusIndicator(		Mutex&						aMutex		,
																		OStatusIndicatorFactory*	aFactory	)
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &aMutex	==	NULL	)	||
			( aFactory	==	NULL	)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool OStatusIndicator::impldbg_checkParameter_start(	const	OUString&	sText	,
																	sal_Int32	nRange	)
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &sText	==	NULL	)	||
			( &nRange	==	NULL	)	||
			( nRange	<	1		)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool OStatusIndicator::impldbg_checkParameter_setText( const OUString& sText )
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &sText == NULL )
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool OStatusIndicator::impldbg_checkParameter_setValue( sal_Int32 nValue, sal_Int32 nRange )
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			(	nValue	<	0		)	||
			(	nValue	>	nRange	)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

#endif	//	#ifdef ENABLE_ASSERTIONS

}		//	namespace framework
