/*************************************************************************
 *
 *  $RCSfile: InteractionRequest.cxx,v $
 *
 *  $Revision: 1.1 $
 *
 *  last change: $Author: mtg $ $Date: 2001/07/04 14:56:37 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _INTERACTION_REQUEST_HXX_
#include <InteractionRequest.hxx>
#endif
#ifndef _INTERACTION_CONTINUATION_HXX_
#include <InteractionContinuation.hxx>
#endif

using namespace ::rtl;
using namespace ::com::sun::star::task;
using namespace ::com::sun::star::uno;

InteractionRequest::InteractionRequest ( const Any& rRequest, sal_uInt16 nResponseFlags )
: maRequest( rRequest )
{
	sal_Int32 nLength = 0;
	if( nResponseFlags & EC_YES ) nLength++;
	if( nResponseFlags & EC_NO ) nLength++;
	if( nResponseFlags & EC_RETRY ) nLength++;
	if( nResponseFlags & EC_ABORT ) nLength++;

	maContinuations.realloc( nLength );
	Reference< XInteractionContinuation >* pContinuations = maContinuations.getArray();

	if( nResponseFlags & EC_YES )
		*pContinuations++ = new InteractionApprove();

	if( nResponseFlags & EC_NO )
		*pContinuations++ = new InteractionDisapprove();

	if( nResponseFlags & EC_RETRY )
		*pContinuations++ = new InteractionRetry();

	if( nResponseFlags & EC_ABORT )
		*pContinuations++ = new InteractionAbort();
}

InteractionRequest::~InteractionRequest ()
{
}

Any SAL_CALL InteractionRequest::getRequest(void) 
	throw(RuntimeException)
{
	return maRequest;
}

Sequence< Reference< XInteractionContinuation > > InteractionRequest::getContinuations(void) 
	throw(RuntimeException)
{
	return maContinuations;
}

sal_uInt16 InteractionRequest::getSelection() const
{
	sal_Int32 nLength = maContinuations.getLength();
	const Reference< XInteractionContinuation >* pContinuations = maContinuations.getConstArray();

	while( nLength-- )
	{
		InteractionContinuation* pCont = InteractionContinuation::getImplementation( *pContinuations++ );
		
		if( pCont && pCont->isSelected() )
			return pCont->getErrorContinuation();
	}

	return 0;
}
// XInterface
Any SAL_CALL InteractionRequest::queryInterface( const Type& rType ) 
	throw(RuntimeException)
{
		return ::cppu::queryInterface ( rType,
										// OWeakObject interfaces
										reinterpret_cast< XInterface*		> ( this ),
										static_cast< XWeak*					> ( this ),
										// my own interfaces
							   		 	static_cast< XInteractionRequest*	> ( this ) );
}

void SAL_CALL InteractionRequest::acquire(  ) 
	throw()
{
	OWeakObject::acquire();
}
void SAL_CALL InteractionRequest::release(  ) 
	throw()
{
	OWeakObject::release();
}
