/*************************************************************************
 *
 *  $RCSfile: readchrt.cxx,v $
 *
 *  $Revision: 1.9 $
 *
 *  last change: $Author: dr $ $Date: 2001/07/24 13:49:15 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifdef PCH
#include "filt_pch.hxx"
#endif

#pragma hdrstop

//------------------------------------------------------------------------

#ifndef _SCERRORS_HXX
#include "scerrors.hxx"
#endif

#ifndef _SC_XCLIMPSTREAM_HXX
#include "XclImpStream.hxx"
#endif
#ifndef _IMP_OP_HXX
#include "imp_op.hxx"
#endif
#ifndef _EXCIMP8_HXX
#include "excimp8.hxx"
#endif
#ifndef _FLTPRGRS_HXX
#include "fltprgrs.hxx"
#endif

FltError ImportExcel8::ReadChart8( FilterProgressBar& rPrgrs, const BOOL bOwnTab )
{
	bFirstScl	= TRUE;

	FltError	eLastErr = eERR_OK;
	BOOL		bLoop;
	UINT16		nOpcode;			// current opcode
	pActChart = GetActChartData();

	if( !pActChart )
	{
		bLoop = TRUE;
		while( bLoop )
		{
			bLoop = aIn.StartNextRecord();
			if( bLoop )
				bLoop = (aIn.GetRecNum() != 0x000A);
		}
		rPrgrs.Progress();

		return eERR_OK;
	}

	bLoop = TRUE;
	while( bLoop )
	{
		bLoop = aIn.StartNextRecord();
		nOpcode = aIn.GetRecNum();

		rPrgrs.Progress();

		switch( nOpcode )
		{
            case 0x000A:    ChartEof(); bLoop = FALSE;          break;  // EOF
			case 0x0014:	Header();							break;	// HEADER
			case 0x0015:	Footer();							break;	// FOOTER
			case 0x0026:	Leftmargin();						break;	// LEFTMARGIN
			case 0x0027:	Rightmargin();						break;	// RIGHTMARGIN
			case 0x0028:	Topmargin();						break;	// TOPMARGIN
			case 0x0029:	Bottommargin();						break;	// BOTTOMMARGIN
			case 0x002A:	Printheaders();						break;	// PRINTHEADERS
			case 0x00A0:	ChartScl();							break;	// SCL
			case 0x00A1:	if( bOwnTab ) Setup();				break;	// SETUP
			case 0x1002:	ChartChart();						break;	// CHART
			case 0x1003:	ChartSeries();						break;	// SERIES
			case 0x1006:	ChartDataformat();					break;	// DATAFORMAT
			case 0x1007:	ChartLineformat();					break;	// LINEFORMAT
			case 0x1009:	ChartMarkerformat();				break;	// MARKERFORMAT
			case 0x100A:	ChartAreaformat();					break;	// AREAFORMAT
			case 0x100B:	ChartPieformat();					break;	// PIEFORMAT
			case 0x100C:	ChartAttachedlabel();				break;	// ATTACHEDLABEL
			case 0x100D:	ChartSeriestext();					break;	// SERIESTEXT
			case 0x1014:	ChartChartformat();					break;	// CHARTFORMAT
			case 0x1015:	ChartLegend();						break;	// LEGEND
			case 0x1017:	ChartBar();							break;	// BAR
			case 0x1018:	ChartLine();						break;	// LINE
			case 0x1019:	ChartPie();							break;	// PIE
			case 0x101A:	ChartArea();						break;	// AREA
			case 0x101B:	ChartScatter();						break;	// SCATTER
			case 0x101C:	ChartChartline();					break;	// CHARTLINE
			case 0x101D:	ChartAxis();						break;	// AXIS
			case 0x101E:	ChartTick();						break;	// TICK
			case 0x101F:	ChartValuerange();					break;	// VALUERANGE
			case 0x1020:	ChartCatserrange();					break;	// CATSERRANGE
			case 0x1021:	ChartAxislineformat();				break;	// AXISLINEFORMAT
			case 0x1024:	ChartDefaulttext();					break;	// DEFAULTTEXT
			case 0x1025:	ChartText();						break;	// TEXT
			case 0x1026:	ChartFontx();						break;	// FONTX
			case 0x1027:	ChartObjectlink();					break;	// OBJECTLINK
			case 0x1032:	ChartFrame();						break;	// FRAME
			case 0x1033:	ChartBegin();						break;	// BEGIN
			case 0x1034:	ChartEnd();							break;	// END
			case 0x1035:	ChartPlotarea();					break;	// PLOTAREA
			case 0x103A:	Chart3D();							break;	// 3D
			case 0x103D:	ChartDropbar();						break;	// DROPBAR
			case 0x103E:	ChartRadar();						break;	// RADAR
			case 0x103F:	ChartSurface();						break;	// SURFACE
			case 0x1041:	ChartAxisparent();					break;	// AXISPARENT
			case 0x1044:	ChartShtprops();					break;	// SHTPROPS
			case 0x1045:	ChartSertocrt();					break;	// SERTOCRT
			case 0x1046:	ChartAxesused();					break;	// AXESUSED
			case 0x104E:	ChartIfmt();						break;	// IFMT
			case 0x104F:	ChartPos();							break;	// POS
			case 0x1051:	ChartAi();							break;	// AI
			case 0x105D:	ChartSerfmt();						break;	// SERFMT
			case 0x105F:	Chart3DDataformat();				break;	// 3DDATAFORMAT
			case 0x1060:	ChartFbi();							break;	// FBI
			case 0x1062:	ChartAxcext();						break;	// AXCEXT
			case 0x1064:	ChartPlotgrowth();					break;	// PLOTGROWTH
			case 0x1066:	ChartGelframe();					break;	// GELFRAME
		}
	}

	return eLastErr;
}

