/*************************************************************************
 *
 *  $RCSfile: XclExpExternsheet.hxx,v $
 *
 *  $Revision: 1.1 $
 *
 *  last change: $Author: dr $ $Date: 2001/07/30 11:33:30 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SC_XCLEXPETERNSHEET_HXX
#define _SC_XCLEXPETERNSHEET_HXX

#ifndef _LIST_HXX
#include <tools/list.hxx>
#endif

#ifndef _FLTTOOLS_HXX
#include "flttools.hxx"
#endif
#ifndef _EXCRECDS_HXX
#include "excrecds.hxx"
#endif

//___________________________________________________________________
// 3D references, external references
//  --> records TABID, SUPBOOK, XCT, CRN, EXTERNNAME, EXTERNSHEET

//___________________________________________________________________
// stores the correct Excel table number for each Calc table

class XclExpTabNumBuffer : private UINT16List
{
private:
    UINT32*                     pBuffer;

    UINT16                      nScCnt;
    UINT16                      nExcCnt;
    UINT16                      nExtCnt;
    UINT16                      nCodeCnt;

    BOOL                        bEnableLog;

    void                        ApplyBuffer();

protected:
public:
                                XclExpTabNumBuffer( ScDocument& rDoc );
                                ~XclExpTabNumBuffer();

    BOOL                        IsExternal( UINT16 nScTab ) const;
    BOOL                        IsExportTable( UINT16 nScTab ) const;
    UINT16                      GetExcTable( UINT16 nScTab ) const;
    inline UINT16               GetScTabCount() const       { return nScCnt; }
    inline UINT16               GetExcTabCount() const      { return nExcCnt; }
    inline UINT16               GetExternTabCount() const   { return nExtCnt; }
    inline UINT16               GetCodenameCount() const    { return nCodeCnt; }
    inline UINT16               GetMaxScTabCount() const    { return Max( nScCnt, nCodeCnt ); }

// for change tracking:
                                // append table number pair, called by formula compiler
    void                        AppendTabRef( UINT16 nExcFirst, UINT16 nExcLast );
                                // enables logging of excel table nums in every 3D-ref
    inline void                 StartRefLog()       { UINT16List::Clear(); bEnableLog = TRUE; }
                                // disables logging
    inline void                 EndRefLog()         { bEnableLog = FALSE; }
                                // returns the index list
    inline const UINT16List&    GetRefLog() const   { return *this; }
};


//___________________________________________________________________
// base class: stores the contents of one external cell

class XclExpCrn : public ExcRecord
{
private:
    UINT16                      nCol;
    UINT16                      nRow;
    UINT8                       nId;

protected:
    virtual void                SaveCont( XclExpStream& rStrm );
    virtual void                SaveDiff( XclExpStream& rStrm ) = 0;
    virtual ULONG               GetDiffLen() const = 0;

public:
    inline                      XclExpCrn( UINT16 _nCol, UINT16 _nRow, UINT8 _nId ) :
                                    nCol( _nCol ), nRow( _nRow ), nId( _nId ) {}

    inline BOOL                 IsAddress( UINT16 _nCol, UINT16 _nRow )
                                    { return (_nCol == nCol) && (_nRow == nRow); }

    virtual UINT16              GetNum() const;
    virtual ULONG               GetLen() const;
};


//___________________________________________________________________
// external value cell

class XclExpCrnDouble : public XclExpCrn
{
private:
    double                      fVal;
    virtual void                SaveDiff( XclExpStream& rStrm );
    virtual ULONG               GetDiffLen() const;

protected:
public:
                                XclExpCrnDouble( UINT16 _nCol, UINT16 _nRow, double _fVal );
};


//___________________________________________________________________
// external string cell

class XclExpCrnString : public XclExpCrn
{
private:
    XclExpUniString             aText;
    virtual void                SaveDiff( XclExpStream& rStrm );
    virtual ULONG               GetDiffLen() const;

protected:
public:
                                XclExpCrnString( UINT16 _nCol, UINT16 _nRow, const String& rTxt );
};


//___________________________________________________________________
// external boolean cell

class XclExpCrnBool : public XclExpCrn
{
private:
    UINT16                      nBool;
    virtual void                SaveDiff( XclExpStream& rStrm );
    virtual ULONG               GetDiffLen() const;

protected:
public:
                                XclExpCrnBool( UINT16 _nCol, UINT16 _nRow, BOOL bBoolVal );
};


//___________________________________________________________________
// header record for CRN record list

class XclExpXct : public ExcRecord, private List
{
private:
    XclExpUniString             aTable;
    UINT16                      nTabNum;

    inline XclExpCrn*           _First()    { return (XclExpCrn*) List::First(); }
    inline XclExpCrn*           _Next()     { return (XclExpCrn*) List::Next(); }

    BOOL                        Exists( UINT16 nCol, UINT16 nRow );
    inline void                 Append( XclExpCrn* pCrn )
                                    { List::Insert( pCrn, LIST_APPEND ); }

    virtual void                SaveCont( XclExpStream& rStrm );

protected:
public:
    inline                      XclExpXct( const String& rTab ) :
                                    aTable( rTab )  {}
    virtual                     ~XclExpXct();

    inline UINT16               GetTableBytes() const   { return (UINT16) aTable.GetByteCount(); }
    inline const XclExpUniString& GetTableName() const  { return aTable; }

    inline void                 SetTableNum( UINT16 nTab )  { nTabNum = nTab; }
    void                        StoreCellRange( RootData& rRootData, const ScRange& rRange );

    virtual void                Save( XclExpStream& rStrm );

    virtual UINT16              GetNum() const;
    virtual ULONG               GetLen() const;
};


//___________________________________________________________________
// external name (base class for AddIn- and DDE EXTERNNAMEs)

class XclExpExternname : public ExcRecord
{
private:
    String                      aName;
    UINT16                      nFlags;

protected:
    virtual void                SaveCont( XclExpStream& rStrm );
    virtual void                SaveDiff( XclExpStream& rStrm ) = 0;
    virtual ULONG               GetDiffLen() const = 0;

public:
                                XclExpExternname( const String& rName, UINT16 _nFlags );
    virtual                     ~XclExpExternname();

    inline const String&        GetName() const     { return aName; }

    virtual UINT16              GetNum() const;
    virtual ULONG               GetLen() const;
};


//___________________________________________________________________
// external name for an AddIn function

class XclExpExternnameAddIn : public XclExpExternname
{
private:
    virtual void                SaveDiff( XclExpStream& rStrm );
    virtual ULONG               GetDiffLen() const;

public:
    inline                      XclExpExternnameAddIn( const String& rName ) :
                                    XclExpExternname( rName, 0x0000 ) {}
};


//___________________________________________________________________
// external name for a DDE link

class XclExpExternnameDDE : public XclExpExternname
{
private:
    XclExpCachedValueList*      pOpList;

    virtual void                SaveDiff( XclExpStream& rStrm );
    virtual ULONG               GetDiffLen() const;

public:
    inline                      XclExpExternnameDDE( const String& rName, UINT16 _nFlags ) :
                                    XclExpExternname( rName, _nFlags ), pOpList( NULL ) {}
    virtual                     ~XclExpExternnameDDE();

    BOOL                        InsertDDE(
                                    RootData& rRootData,
                                    const String& rApplic,
                                    const String& rTopic,
                                    const String& rItem );
};


//___________________________________________________________________
// list of all external names

class XclExpExternnameList : private List
{
private:
    inline XclExpExternname*    _First()    { return (XclExpExternname*) List::First(); }
    inline XclExpExternname*    _Next()     { return (XclExpExternname*) List::Next(); }

    UINT16                      GetExtname( const String& rName );

    inline UINT16               Append( XclExpExternname* pName );

public:
    virtual                     ~XclExpExternnameList();

                                // these functions return the Xcl index of
                                // the EXTERNNAME record (1-based)
    UINT16                      InsertAddin( const String& rName );
    UINT16                      InsertDDE(
                                    RootData& rRootData,
                                    const String& rApplic,
                                    const String& rTopic,
                                    const String& rItem );

    void                        Save( XclExpStream& rStrm );
};

inline UINT16 XclExpExternnameList::Append( XclExpExternname* pName )
{
    List::Insert( pName, LIST_APPEND );
    return (UINT16) List::Count();
}


//___________________________________________________________________
// data for an external document (doc name, table names, external values)

class XclExpSupbook : public ExcRecord, private List
{
private:
    enum Type                   { SB_Addin, SB_Self, SB_Ext };

    String                      aDocName;
    XclExpUniString             aEncoded;
    XclExpExternnameList*       pExtnameList;
    Type                        eType;
    ULONG                       nLen;
    UINT16                      nTables;

    inline XclExpXct*           _First()    { return (XclExpXct*) List::First(); }
    inline XclExpXct*           _Next()     { return (XclExpXct*) List::Next(); }
    inline XclExpXct*           _Get( UINT16 nTab ) const
                                            { return (XclExpXct*) List::GetObject( nTab ); }
    XclExpExternnameList&       GetExternnameList();

    virtual void                SaveCont( XclExpStream& rStrm );

public:
                                XclExpSupbook();                                        // Addins
                                XclExpSupbook( UINT16 nTabs );                          // own book
                                XclExpSupbook( const String& rDocName, BOOL bConvert ); // ext book
    virtual                     ~XclExpSupbook();

                                // original document name
    inline const String&        GetName() const         { return aDocName; }
                                // Excel encoded document name
    inline const XclExpUniString& GetEncName() const    { return aEncoded; }
                                // table name inside of this supbook
    const XclExpUniString*      GetTableName( UINT16 nIndex ) const;

    inline BOOL                 IsSelf() const          { return eType == SB_Self; }
    inline BOOL                 IsAddin() const         { return eType == SB_Addin; }
    inline BOOL                 IsExt() const           { return eType == SB_Ext; }

                                // cell contents -> CRN
    void                        StoreCellRange( RootData& rRootData, const ScRange& rRange, UINT16 nXct );

                                // name of an external table
    UINT16                      InsertTable( const String& rTabName );
                                // create or get Addin EXTERNNAME index
    UINT16                      InsertAddin( const String& rName );
                                // create or get DDE EXTERNNAME index
    UINT16                      InsertDDE(
                                    RootData& rRootData,
                                    const String& rApplic,
                                    const String& rTopic,
                                    const String& rItem );

    virtual void                Save( XclExpStream& rStrm );

    virtual UINT16              GetNum() const;
    virtual ULONG               GetLen() const;
};


//___________________________________________________________________
// list of all supbooks

class XclExpSupbookList : public ExcEmptyRec, public ExcRoot, private List
{
private:
    UINT16*                     pSupbookBuffer; // supbook number for every xcl table
    UINT16*                     pTableBuffer;   // table numbers in supbook
    UINT16                      nRefdCnt;       // array size for p***Buffer
    UINT16                      nAddinSupb;     // index to Addin supbook

    inline XclExpSupbook*       _First()    { return (XclExpSupbook*) List::First(); }
    inline XclExpSupbook*       _Next()     { return (XclExpSupbook*) List::Next(); }
    inline XclExpSupbook*       _Get( ULONG nIndex )
                                            { return (XclExpSupbook*) List::GetObject( nIndex ); }

    inline XclExpSupbook*       GetSupbook( UINT16 nExcTab ) const;
    XclExpSupbook*              GetSupbook( UINT16& rnIndex, const String& rName );

    UINT16                      Append( XclExpSupbook* pBook );
    void                        AddExt( UINT16 nScTab );

public:
                                XclExpSupbookList( RootData& rRootData );
    virtual                     ~XclExpSupbookList();

                                // get external document name - expects Excel tabnums
    const XclExpUniString*      GetDocumentName( UINT16 nExcTab ) const;
                                // get external table name - expects Excel tabnums
    const XclExpUniString*      GetTableName( UINT16 nExcTab ) const;
                                // get supbook index and table range - expects Excel tabnums
    void                        GetXtiRange(
                                    UINT16& rnSupb, UINT16& rnXtiFirst, UINT16& rnXtiLast,
                                    UINT16 nTabFirst, UINT16 nTabLast ) const;

                                // cell contents -> CRN
    void                        StoreCellRange( const ScRange& rRange );

                                // create or get Addin EXTERNNAME index
    void                        InsertAddin(
                                    UINT16& rnSupbook, UINT16& rnExtname,
                                    const String& rName );
                                // create or get DDE EXTERNNAME index
    void                        InsertDDE(
                                    UINT16& rnSupbook, UINT16& rnExtname,
                                    const String& rApplic,
                                    const String& rTopic,
                                    const String& rItem );

    virtual void                Save( XclExpStream& rStrm );
};

inline XclExpSupbook* XclExpSupbookList::GetSupbook( UINT16 nExcTab ) const
{
    DBG_ASSERT( nExcTab < nRefdCnt, "XclSupbookList::GetSupbook - out of range" );
    return (XclExpSupbook*) List::GetObject( pSupbookBuffer[ nExcTab ] );
}


//___________________________________________________________________
// table index range for 3D references
// stores index to supbook and indexes to supbook table names

struct XclExpXti
{
    UINT16                      nSupb;
    UINT16                      nFirst;
    UINT16                      nLast;

    inline                      XclExpXti( UINT16 nSupbook, UINT16 nTabFirst, UINT16 nTabLast ) :
                                    nSupb( nSupbook ), nFirst( nTabFirst ), nLast( nTabLast ) {}

    inline BOOL                 Equals(  UINT16 nSupbook, UINT16 nTabFirst, UINT16 nTabLast ) const
                                    {return (nSupb == nSupbook) && (nFirst == nTabFirst) && (nLast == nTabLast); }

    inline void                 Save( XclExpStream& rStrm ) const
                                    { rStrm << nSupb << nFirst << nLast; }
};


//___________________________________________________________________
// list of table indexes of all used 3D references

class XclExpExternsheetList : public ExcRecord, public ExcRoot, private List
{
private:
    XclExpSupbookList           aSupbookList;

    inline XclExpXti*           _First()    { return (XclExpXti*) List::First(); }
    inline XclExpXti*           _Next()     { return (XclExpXti*) List::Next(); }
    inline const XclExpXti*     _Get( ULONG nIndex )
                                    { return (const XclExpXti*) List::GetObject( nIndex ); }

    UINT16                      Append( XclExpXti* pXti );
    UINT16                      InsertXti( UINT16 nSupb, UINT16 nFirst, UINT16 nLast );

                                // Xcl design error
    static inline UINT16        GetVal16( ULONG nVal )
                                    { return (UINT16) Min( nVal, 0xFFFFUL ); }
    inline UINT16               GetCount16() const
                                    { return GetVal16( List::Count() ); }

    virtual void                SaveCont( XclExpStream& rStrm );

public:
                                XclExpExternsheetList( RootData& rRootData );
    virtual                     ~XclExpExternsheetList();

                                // add new XclExpXti if not found - expects Excel tabnums
    UINT16                      Find( UINT16 nTabFirst, UINT16 nTabLast );
                                // add new XclExpXti if not found - expects supbook index
    UINT16                      FindSpecial( UINT16 nSupb );
                                // get external document name - expects Excel tabnums
    inline const XclExpUniString* GetDocumentName( UINT16 nExcTab )
                                    { return aSupbookList.GetDocumentName( nExcTab ); }
                                // get external table name - expects Excel tabnums
    inline const XclExpUniString* GetTableName( UINT16 nExcTab )
                                    { return aSupbookList.GetTableName( nExcTab ); }

                                // cell contents -> CRN - expects Calc tabnums
    void                        StoreCellCont( const SingleRefData& rRef );
    void                        StoreCellRange( const SingleRefData& rRef1, const SingleRefData& rRef2 );

                                // create or get Addin EXTERNNAME index
    void                        InsertAddin(
                                    UINT16& rnXti, UINT16& rnExtname,
                                    const String& rName );
                                // create or get DDE EXTERNNAME index
    BOOL                        InsertDDE(
                                    UINT16& rnXti, UINT16& rnExtname,
                                    const String& rApplic,
                                    const String& rTopic,
                                    const String& rItem );

    virtual void                Save( XclExpStream& rStrm );

    virtual UINT16              GetNum() const;
    virtual ULONG               GetLen() const;
};


//___________________________________________________________________

#endif

