/*************************************************************************
 *
 *  $RCSfile: XclImpExternsheet.hxx,v $
 *
 *  $Revision: 1.3 $
 *
 *  last change: $Author: dr $ $Date: 2001/07/30 11:25:46 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/



#ifndef _SC_XCLIMPEXTERNSHEET_HXX
#define _SC_XCLIMPEXTERNSHEET_HXX

#ifndef _LIST_HXX
#include <tools/list.hxx>
#endif

#ifndef _EXCDEFS_HXX
#include "excdefs.hxx"
#endif
#ifndef _FLTTOOLS_HXX
#include "flttools.hxx"
#endif

class ScDocument;
class ScTokenArray;
class XclImpStream;
class ExcelToSc;
struct RootData;


//___________________________________________________________________
// 3D references, external references
//  --> records TABID, SUPBOOK, XCT, CRN, EXTERNNAME, EXTERNSHEET

//___________________________________________________________________
// order of the sheets inside of the workbook

class XclImpTabIdBuffer : protected UINT16List
{
private:
	void						Append( UINT16 nTabId );

public:
	void						Fill( XclImpStream& rStrm, UINT16 nCount );
	UINT16						GetIndex( UINT16 nTabId, UINT16 nMaxTabId = 0xFFFF ) const;
};


//___________________________________________________________________
// external names (named ranges, AddIn names, DDE links, OLE links)

enum XclImpExtNameType          { xienName, xienDDE, xienOLE };

class XclImpExtName
{
private:
    String                      aName;
    UINT32                      nStorageId;
    XclImpExtNameType           eType;

public:
                                XclImpExtName(
                                    XclImpExtNameType _eType,
                                    const String& rName,
                                    UINT32 _nStorageId = 0 );

    inline XclImpExtNameType    GetType() const         { return eType; }
    inline const String&        GetName() const         { return aName; }
    inline UINT32               GetStorageId() const    { return nStorageId; }
};


//___________________________________________________________________
// buffer for external names of one supbook

class XclImpExtNameBuffer : private List
{
private:
    inline XclImpExtName*       _First()    { return (XclImpExtName*) List::First(); }
    inline XclImpExtName*       _Next()     { return (XclImpExtName*) List::Next(); }
    inline void                 Append( XclImpExtName* pName )
                                    { List::Insert( pName, LIST_APPEND ); }

public:
    inline                      XclImpExtNameBuffer() {}
    virtual                     ~XclImpExtNameBuffer();

    void                        AddName( const String& rName, RootData& rRootData );
    inline void                 AddDDE( const String& rName );
    inline void                 AddOLE( const String& rName, UINT32 nStorageId );

    const XclImpExtName*        GetName( UINT16 nXclIndex ) const;
};

inline void XclImpExtNameBuffer::AddDDE( const String& rName )
{
    Append( new XclImpExtName( xienDDE, rName ) );
}

inline void XclImpExtNameBuffer::AddOLE( const String& rName, UINT32 nStorageId )
{
    Append( new XclImpExtName( xienOLE, rName, nStorageId ) );
}


//___________________________________________________________________
// cached cell contents

class XclImpCrnBase
{
protected:
	USHORT						nCol;
	USHORT						nRow;

	inline						XclImpCrnBase( USHORT _nCol, USHORT _nRow ) :
									nCol( _nCol ), nRow( _nRow ) {}

public:
	virtual						~XclImpCrnBase();
	virtual void				SetCell( ScDocument* pDoc, USHORT nTab ) = 0;
};



class XclImpCrnDouble : public XclImpCrnBase
{
protected:
	double						fValue;

public:
	inline						XclImpCrnDouble( USHORT nCol, USHORT nRow, double fVal ) :
									XclImpCrnBase( nCol, nRow ), fValue( fVal ) {}

	virtual void				SetCell( ScDocument* pDoc, USHORT nTab );
};



class XclImpCrnString : public XclImpCrnBase
{
protected:
	String						aText;

public:
	inline						XclImpCrnString( USHORT nCol, USHORT nRow, const String& rText ) :
									XclImpCrnBase( nCol, nRow ), aText( rText ) {}

	virtual void				SetCell( ScDocument* pDoc, USHORT nTab );
};



class XclImpCrnFormula : public XclImpCrnDouble
{
protected:
	const ScTokenArray*			pTokArr;

public:
                                XclImpCrnFormula(
                                    USHORT nCol, USHORT nRow, double fVal,
                                    const ScTokenArray* pTokenArray );
    virtual                     ~XclImpCrnFormula();

	virtual void				SetCell( ScDocument* pDoc, USHORT nTab );
};


//___________________________________________________________________
// a referenced table inside of an external workbook

class XclImpSupbookTab : private List
{
private:
	String						aName;
	UINT16						nScTab;

	inline XclImpCrnBase*		_First()	{ return (XclImpCrnBase*) List::First(); }
	inline XclImpCrnBase*		_Next()		{ return (XclImpCrnBase*) List::Next(); }

public:
	inline						XclImpSupbookTab( const String& rName ) :
									aName( rName ), nScTab( EXC_TAB_INVALID ) {}
	virtual						~XclImpSupbookTab();

	inline const String&		GetName() const					{ return aName; }
	inline UINT16				GetScTab() const				{ return nScTab; }

	inline void					SetScTab( UINT16 nTab )			{ nScTab = nTab; }

	inline void					AppendCrn( XclImpCrnBase* pCrn )
									{ if( pCrn ) List::Insert( pCrn, LIST_APPEND ); }

	void						CreateTable( ScDocument* pDoc, const String& rURL );
};


//___________________________________________________________________
// list of all referenced tables inside of an external workbook

class XclImpSupbook : private List
{
private:
    XclImpExtNameBuffer         aExtNameBuffer;
    String                      aFileName;
    UINT16                      nCurrExcTab;
    BOOL                        bSelf;

    inline XclImpSupbookTab*    _First()    { return (XclImpSupbookTab*) List::First(); }
    inline XclImpSupbookTab*    _Next()     { return (XclImpSupbookTab*) List::Next(); }
    inline XclImpSupbookTab*    _Get( ULONG nIndex ) const
                                    { return (XclImpSupbookTab*) List::GetObject( nIndex ); }

    inline void                 Append( XclImpSupbookTab* pTab )
                                    { List::Insert( pTab, LIST_APPEND ); }
    void                        AppendCrn( XclImpCrnBase*& rpCrn );

public:
                                XclImpSupbook( XclImpStream& rStrm );
    virtual                     ~XclImpSupbook();

    static void                 ReadDocName( XclImpStream& rStrm, String& rDocName, BOOL& rbSelf );
    static void                 ReadTabName( XclImpStream& rStrm, String& rTabName );

    inline BOOL                 IsShortened() const     { return (aFileName.Len() == 0); }
    inline BOOL                 IsSelf() const          { return bSelf; }
    inline const String&        GetName() const         { return aFileName; }
    inline const XclImpExtName* GetExtName( UINT16 nXclIndex ) const;
    BOOL                        GetLink( String& rApplic, String& rDoc ) const;

    UINT16                      GetScTabNum( UINT16 nExcTabNum ) const;
    UINT16                      GetScTabNum( const String& rTabName ) const;

    void                        ReadXct( XclImpStream& rStrm );
    void                        ReadCrn( XclImpStream& rStrm, ExcelToSc* pFmlConv );
    void                        ReadExternname( XclImpStream& rStrm, RootData& rRootData );

    void                        CreateTables( RootData& rRootData, UINT16 nFirst, UINT16 nLast ) const;
};

inline const XclImpExtName* XclImpSupbook::GetExtName( UINT16 nXclIndex ) const
{
    return aExtNameBuffer.GetName( nXclIndex );
}


//___________________________________________________________________
// list of all supbooks

class XclImpSupbookBuffer : private List
{
private:
	inline XclImpSupbook*		_First()	{ return (XclImpSupbook*) List::First(); }
	inline XclImpSupbook*		_Next()		{ return (XclImpSupbook*) List::Next(); }
	inline XclImpSupbook*		_Last()		{ return (XclImpSupbook*) List::Last(); }

public:
	virtual						~XclImpSupbookBuffer();

	inline void					Append( XclImpSupbook* pNewSupbook )
									{ List::Insert( pNewSupbook, LIST_APPEND ); }
	void						CreateTables( RootData& rRootData, UINT16 nFirst, UINT16 nLast );

								List::Count;
	inline const XclImpSupbook*	Get( ULONG nIndex ) const	{ return (XclImpSupbook*) List::GetObject( nIndex ); }
	const XclImpSupbook*		Get( const String& rDocName ) const;
	inline XclImpSupbook*		GetCurrSupbook() { return _Last(); }
};


//___________________________________________________________________

struct XclImpXti
{
	UINT16						nSupbook;		// index to supbook
	UINT16						nFirst;			// first sheet tab
	UINT16						nLast;			// last sheet tab
};


//___________________________________________________________________
// stores all EXTERNSHEET and SUPBOOK data

class XclImpExternsheetBuffer : private List
{
private:
	XclImpSupbookBuffer			aSupbookBuffer;
	BOOL						bCreated;

	inline XclImpXti*			_First()	{ return (XclImpXti*) List::First(); }
	inline XclImpXti*			_Next()		{ return (XclImpXti*) List::Next(); }

	BOOL						FindNextTabRange( UINT16 nSupb, UINT16 nStart, UINT16& rnFirst, UINT16& rnLast );

public:
	inline						XclImpExternsheetBuffer() : bCreated( FALSE ) {}
	virtual						~XclImpExternsheetBuffer();

	void						Read( XclImpStream& rIn );
	inline void					AppendSupbook( XclImpSupbook* pSupbook )	{ aSupbookBuffer.Append( pSupbook ); }
	void						CreateTables( RootData& rRootData );

	inline const XclImpXti*		GetXti( ULONG nXtiIndex ) const	{ return (const XclImpXti*) List::GetObject( nXtiIndex ); }

	const XclImpSupbook*		GetSupbook( ULONG nXtiIndex ) const;
	inline const XclImpSupbook*	GetSupbook( const String& rDocName ) const	{ return aSupbookBuffer.Get( rDocName ); }
	inline XclImpSupbook*		GetCurrSupbook()	{ return aSupbookBuffer.GetCurrSupbook(); }
};


//___________________________________________________________________

#endif

