/*************************************************************************
 *
 *  $RCSfile: red.cxx,v $
 *
 *  $Revision: 1.3 $
 *
 *  last change: $Author: ok $ $Date: 2000/11/28 11:09:06 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRUNTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRUNTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc..
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifdef UNX
#include <unistd.h>
#endif

#include <stdio.h>

#ifndef _SV_SVAPP_HXX
#include <vcl/svapp.hxx>
#endif

#ifndef _FSYS_HXX
#include <tools/fsys.hxx>
#endif

#ifndef _FSYS_HXX
#include <tools/fsys.hxx>
#endif

#include "rtl/crc.h"
#include "setup2/sifsys.hxx"
#include "setup2/hashtbl.hxx"

#define MAX_BUF_SIZE		32000

ByteString	aSourcePath;
BOOL		bTestOnlyMode = FALSE;
BOOL		bVerboseMode  = FALSE;

// ============================================================================

void PrintRedundancyEliminatorUsage()
{
	fprintf( stderr, "usage:\n" );
	fprintf( stderr, "\t-s <source path root>\n" );
	fprintf( stderr, "\t-t <test only mode>\n" );
	fprintf( stderr, "\t-v verbose\n" );
}

// ============================================================================

class RedundancyEliminator : public Application
{
public:
	virtual void Main() {}
};

RedundancyEliminator aApp;

// ============================================================================

struct FileEntry
{
	sal_uInt32 nCRC;
	sal_uInt32 nSize;
	ByteString aAbsFilename;
	BOOL bReplaced;
};

DECLARE_LIST( Filelist, FileEntry* );

Filelist aFilelist;

// ============================================================================

void ReadSource( const ByteString& rPath )
{
	Dir aDir( rPath, FSYS_KIND_FILE | FSYS_KIND_DIR );
	for( USHORT i = 0; i < aDir.Count(); ++i )
	{
		SiDirEntry aEntry( aDir[i] );

		if( aEntry.GetName().CompareIgnoreCaseToAscii(".")  == COMPARE_EQUAL ||
			aEntry.GetName().CompareIgnoreCaseToAscii("..") == COMPARE_EQUAL )
			continue;

		FileStat aStat( aEntry );
		if( aStat.GetKind() == FSYS_KIND_DIR )
			ReadSource( aEntry.GetFull() );
		else
		{
			if( aEntry.GetName().CompareIgnoreCaseToAscii("f0_", 3) != COMPARE_EQUAL &&
				aEntry.GetName().CompareIgnoreCaseToAscii("f_", 2)  != COMPARE_EQUAL )
				continue;

			FILE* file = fopen( aEntry.GetFull().GetBuffer(), "rb" );
			sal_uInt32 nCRC = 0;

			if( file )
			{
				char buf[MAX_BUF_SIZE];
				int num = 0;
				while( ( num = fread( buf, sizeof(char), MAX_BUF_SIZE, file) ) > 0 )
					nCRC = rtl_crc32(nCRC, buf, num);
				fclose( file );
			}

			FileEntry* pNew = new FileEntry;
			pNew->nCRC = nCRC;
			pNew->nSize = aStat.GetSize();
			pNew->aAbsFilename = aEntry.GetFull();
			pNew->bReplaced = FALSE;

			aFilelist.Insert( pNew, LIST_APPEND );
			if( bVerboseMode )
				fprintf( stderr, "." );
		}
	}
}

// ============================================================================

ULONG Eliminate( USHORT& rCnt )
{
	ULONG nBytes = 0;
	for( USHORT n = 0; n < aFilelist.Count(); ++n )
	{
		FileEntry* pEntry = aFilelist.GetObject(n);

		for( USHORT x = 0; x < aFilelist.Count(); ++x )
		{
			FileEntry* p = aFilelist.GetObject(x);
			if( p->bReplaced || pEntry->bReplaced || p == pEntry )
				continue;
			if( pEntry->nCRC == p->nCRC )
			{
				rCnt++;
				pEntry->bReplaced = TRUE;
				p->bReplaced = TRUE;
				nBytes += p->nSize;
#ifdef UNX
				if( !bTestOnlyMode )
				{
					SiDirEntry aEntry( p->aAbsFilename );
					aEntry.Kill();
					symlink( pEntry->aAbsFilename.GetBuffer(), p->aAbsFilename.GetBuffer() );
				}
#endif
				if( bVerboseMode )
				{
					fprintf( stderr, "%s   ==>   %s\n\n", p->aAbsFilename.GetBuffer(), pEntry->aAbsFilename.GetBuffer() );
				}
			}
		}
	}

	return nBytes;
}

// ============================================================================

int __LOADONCALLAPI main( int argv, char** argc )
{
	fprintf( stderr, "\nRedundancy Eliminator (c) 2000 Sun Microsystems\n\n" );
	if( argv == 1 )
	{
		PrintRedundancyEliminatorUsage();
		exit( -1 );
	}

	// === evaluate parameter & environment ===================================
	for( USHORT n = 1; n < argv; n++ )
	{
		if( argc[ n ][ 0 ] == '-' )
		{
			switch( argc[ n ][ 1 ] )
			{
	   			case 's':	aSourcePath = argc[ ++n ];
							break;
				case 't':	bTestOnlyMode = TRUE;
							break;
				case 'v':	bVerboseMode = TRUE;
							break;
			}
		}
	}

	if( !aSourcePath.Len() )
	{
		fprintf( stderr, "error: no source path root defined\n" );
		PrintRedundancyEliminatorUsage();
		exit( -1 );
	}

	fprintf( stderr, "start file scan\n" );

	ReadSource( aSourcePath );

	fprintf( stderr, "%ld files found\nstart eliminating redundances\n", aFilelist.Count() );

	USHORT nCnt = 0;
	ULONG nBytes = Eliminate( nCnt );

	fprintf( stderr, "eliminates %ld MB in %d files", ((nBytes/1024)/1024), nCnt );

	return 0;
}

