/*************************************************************************
 *
 *  $RCSfile: environ.cxx,v $
 *
 *  $Revision: 1.19 $
 *
 *  last change: $Author: ok $ $Date: 2001/07/27 11:13:27 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _LIST_HXX
#include <tools/list.hxx>
#endif
#ifndef _STRING_HXX
#include <tools/string.hxx>
#endif
#ifndef _SISYS_HXX
#include <sifsys.hxx>
#endif

#ifndef _VOS_SECURITY_HXX_
#include <vos/security.hxx>
#endif

#ifndef _SV_SVAPP_HXX
#include <vcl/svapp.hxx>
#endif

#include "environ.hxx"
#include "decltor.hxx"

SiEnvironment::SiEnvironment()
{
	pArchive = NULL;
	ClearEnvironment();
}

void SiEnvironment::ClearEnvironment()
{
	if( pArchive )
		delete pArchive;

	m_bParamDestPath					= FALSE;
	m_bBigMode							= FALSE;
	pArchive							= NULL;
	m_bFirstInstallation				= TRUE;
	m_pActiveModuleSet					= NULL;

	m_bInUpdate							= FALSE;
	m_bUpdateNoVersion					= FALSE;
	m_bUpdateOldVersion    				= FALSE;
	m_bUpgradeOldVersion				= FALSE;
	m_bUpdateOldVersionNoRights			= FALSE;
	m_bUpdateOldVersionWrongLanguage	= FALSE;

	m_bReboot							= FALSE;
	m_bLogout							= FALSE;
	m_bDefuseRestart					= FALSE;
	m_bPropagateWinEnvChange 			= FALSE;

	m_bAutoRepair						= TRUE;
	m_bMigrationAvailalbe				= FALSE;
	m_bDoMigration						= FALSE;
	m_nMigrationSize					= 0;

	m_eInstallType 						= IT_INVALID;
	m_bIsLocal							= TRUE;
	m_bInstallFromNet					= FALSE;
	m_bIsResponseWizard					= FALSE;
	m_bIsResponse						= FALSE;
	m_nResponseStep						= 0;
	m_bVerboseMode						= FALSE;
	m_bIsAutoWorkstation				= FALSE;
	m_bInstallOnRoot					= FALSE;

	ByteString aEmpty;

	m_aSuiteName						= aEmpty;
	m_aProductName						= aEmpty;
	m_aStartPath						= aEmpty;
	m_aSourcePath						= aEmpty;
	m_aDestPath							= aEmpty;
	m_anInstalledPath					= aEmpty;

	m_aUserName							= aEmpty;
	m_aUserFirstName					= aEmpty;
	m_aUserId							= aEmpty;
	m_aUserEmail						= aEmpty;
	m_aCompanyName						= aEmpty;
	m_aStreet							= aEmpty;
	m_aZip								= aEmpty;
	m_aCity								= aEmpty;
	m_aCustomerNr						= aEmpty;
	m_aCountry							= aEmpty;
	m_nCountry							= 0;

	m_nUILanguage						= 0;
	m_nDefLanguage						= 0;

	m_bUIDoNiceLangDefault 				= TRUE;
	m_bHasKHPatched						= FALSE;
	m_bDoOnlineReg						= FALSE;
	m_bNoBasicError						= FALSE;
	m_bUseSystemDir						= FALSE;
	m_bVirtualMode						= FALSE;
	m_bTimeCheck						= TRUE;
	m_bNoRegistration					= FALSE;
	m_bDlgPrecreateDir					= FALSE;
	m_bForceReinstall					= FALSE;
	m_bForceDeinstall					= FALSE;

	m_bHackFlag_1						= FALSE;
	m_bHackFlag_2						= FALSE;
	m_bHackFlag_3						= FALSE;
	m_bHackFlag_4						= FALSE;
	m_bHackFlag_5						= FALSE;

	m_bAuseStreamFlush					= FALSE;

	ULONG i;
	for( i = 0; i < m_aModSetList.Count(); ++i )
		delete m_aModSetList.GetObject(i);
	m_aModSetList.Clear();

	for( i = 0; i < m_aFollowAppList.Count(); ++i )
		delete m_aFollowAppList.GetObject(i);
	m_aFollowAppList.Clear();

	for( i = 0; i < m_aWorkstationList.Count(); ++i )
		delete m_aWorkstationList.GetObject(i);
	m_aWorkstationList.Clear();

	for( i = 0; i < m_aReplacementList.Count(); ++i )
		delete m_aReplacementList.GetObject(i);
	m_aReplacementList.Clear();

	// Languages
	for( i = 0; i < m_aLangContext.Count(); ++i )
		delete m_aLangContext.GetObject(i);
	m_aLangContext.Clear();

	for( i = 0; i < m_aUILangContext.Count(); ++i )
		delete m_aUILangContext.GetObject(i);
	m_aUILangContext.Clear();

	for( i = 0; i < m_aSwitchCtxInstall.Count(); ++i )
		delete m_aSwitchCtxInstall.GetObject(i);
	m_aSwitchCtxInstall.Clear();

	for( i = 0; i < m_aSwitchCtxDelete.Count(); ++i )
		delete m_aSwitchCtxDelete.GetObject(i);
	m_aSwitchCtxDelete.Clear();
}

SiEnvironment::~SiEnvironment()
{
	ClearEnvironment();
}

void SiEnvironment::AddReplacement(const ByteString& rTag, const ByteString& rValue)
{
	ReplacementEntry* pNew = new ReplacementEntry;

	pNew->aTag = rTag;
	pNew->aValue = rValue;

	AddReplacementList( pNew );
}

ByteString _getLangStr( USHORT nLang )
{
	switch( nLang )
	{
		case 1 : return ByteString("en-US");
		case 3 : return ByteString("pt");
		case 7 : return ByteString("ru");
		case 30 : return ByteString("el");
		case 31 : return ByteString("nl");
		case 33 : return ByteString("fr");
		case 34 : return ByteString("es");
		case 39 : return ByteString("it");
		case 45 : return ByteString("da");
		case 46 : return ByteString("sv");
		case 48 : return ByteString("pl");
		case 49 : return ByteString("de");
		case 81 : return ByteString("ja");
		case 82 : return ByteString("ko");
		case 86 : return ByteString("zh-CN");
		case 88 : return ByteString("zh-TW");
		case 90 : return ByteString("tr");
		case 96 : return ByteString("ar");
	}
	return ByteString("");
}

void SiEnvironment::InitReplacement( SiInstallMode eIMode )
{
	for( ULONG i = 0; i < m_aReplacementList.Count(); ++i )
		delete m_aReplacementList.GetObject(i);
	m_aReplacementList.Clear();

	AddReplacement( "<useraddress>", ByteString("") );
	AddReplacement( "<userfirstname>", GetUserFirstName() );
	AddReplacement( "<username>", GetUserName() );
	AddReplacement( "<userid>", GetUserId() );
	AddReplacement( "<customernr>", GetCustomerNr() );
	AddReplacement( "<title>", GetTitle() );
	AddReplacement( "<email>", GetUserEMail() );
	AddReplacement( "<position>", GetPosition() );
	AddReplacement( "<city>", GetCity() );
	AddReplacement( "<fax>", GetFax() );
	AddReplacement( "<telefonwork>", GetTelefonWork() );
	AddReplacement( "<telefonhome>", GetTelefonHome() );
	AddReplacement( "<street>", GetStreet() );
	AddReplacement( "<company>", GetCompanyName() );
	AddReplacement( "<country>", GetCountry() );
	AddReplacement( "<state>", GetState() );
	AddReplacement( "<zip>", GetZip() );
	AddReplacement( "<workpath>", GetWorkPath( eIMode ) );
	AddReplacement( "<progpath>", GetProgPath( eIMode ) );
	AddReplacement( "<ossystem>", OS::GetGUIPath() );

	AddReplacement( "<ssl_encryption>",
			GetCountryIdx() == VEREINIGTE_STAATEN_VON_AMERIKA ||
			GetCountryIdx() == BUNDESREPUBLIK_DEUTSCHLAND ?
			ByteString("2") : ByteString("1") );

	AddReplacement("<productkey>", GetProductKey() );
	AddReplacement("<suitename>", GetSuiteName() );
	AddReplacement("<productname>", GetProductName() );
	AddReplacement("<singleproductname>", GetSingleProductName() );
	AddReplacement("<productversion>", GetProductVersion() );
	AddReplacement("<internalversion>", GetInternalVersion() );

	ByteString aInstLanguages;
	for( i = 0; i < m_aLangContext.Count(); ++i )
	{
		LanguageContext* pCtx = m_aLangContext.GetObject(i);

		if( i ) aInstLanguages += "|";
		aInstLanguages += _getLangStr( pCtx->nLanguage == LANG_DEFAULT? GetDefLanguage() : pCtx->nLanguage );
	}
	AddReplacement( "<sequence_languages>", aInstLanguages );

	::rtl::OUString aConfigDir;
	NAMESPACE_VOS(OSecurity) aSecurity;
	aSecurity.getConfigDir(aConfigDir);
	::rtl::OString oStrConfigDir = ::rtl::OUStringToOString( aConfigDir, RTL_TEXTENCODING_ASCII_US );
	AddReplacement("<configpath>", ByteString(oStrConfigDir));

#if defined(WNT)
	AddReplacement("<osshowextension>", WinOS::SHDoShowExtension()?
			ByteString("1") : ByteString("0") )	;
	AddReplacement("<osdesktop>", WinOS::SHGetSystemDesktopFolderName().Len() && UseSystemDir()?
			ByteString( "file:///{desktop}/" ) : ByteString(""));
	AddReplacement("<ospersonal>", WinOS::SHGetPersonalFolderName().Len() && UseSystemDir()?
			ByteString( "file:///{documents}/" ) : ByteString(""));
	AddReplacement("<osbookmarkdir>", WinOS::SHGetFavoritesFolderName().Len() && UseSystemDir()?
			ByteString( "file:///{bookmarks}/" ) : ByteString(""));
	AddReplacement("<mydocuments>", ByteString(WinOS::SHGetPersonalFolderName(), osl_getThreadTextEncoding()) );
#endif
#if defined(UNX)
	::rtl::OUString aHomeDir;
	aSecurity.getHomeDir(aHomeDir);
	::rtl::OString oStrHomeDir = ::rtl::OUStringToOString( aHomeDir, osl_getThreadTextEncoding() );
	AddReplacement("<mydocuments>", ByteString(oStrHomeDir) );
#endif
}

BOOL SiEnvironment::IsHackFlag(USHORT nFlag) const
{
	switch( nFlag )
	{
		case 1 : return m_bHackFlag_1;
		case 2 : return m_bHackFlag_2;
		case 3 : return m_bHackFlag_3;
		case 4 : return m_bHackFlag_4;
		case 5 : return m_bHackFlag_5;
	}
	return FALSE;
}

void SiEnvironment::SetHackFlag(USHORT nFlag)
{
	switch( nFlag )
	{
		case 1 : m_bHackFlag_1 = TRUE; break;
		case 2 : m_bHackFlag_2 = TRUE; break;
		case 3 : m_bHackFlag_3 = TRUE; break;
		case 4 : m_bHackFlag_4 = TRUE; break;
		case 5 : m_bHackFlag_5 = TRUE; break;
	}
}

void SiEnvironment::ClearHackFlag(USHORT nFlag)
{
	switch( nFlag )
	{
		case 1 : m_bHackFlag_1 = FALSE; break;
		case 2 : m_bHackFlag_2 = FALSE; break;
		case 3 : m_bHackFlag_3 = FALSE; break;
		case 4 : m_bHackFlag_4 = FALSE; break;
		case 5 : m_bHackFlag_5 = FALSE; break;
	}
}

void SiEnvironment::AddSwitchContextInstall(LanguageContext* pToAdd)
{
	for( USHORT i = 0; i < m_aSwitchCtxInstall.Count(); ++i )
	{
		LanguageContext* pCmp = m_aSwitchCtxInstall.GetObject(i);
		if( pCmp->nLanguage == pToAdd->nLanguage	&&
			pCmp->isProg	== pToAdd->isProg		&&
			pCmp->isDoc		== pToAdd->isDoc )
			return;
	}
	m_aSwitchCtxInstall.Insert(pToAdd,LIST_APPEND);
}

void SiEnvironment::AddSwitchContextDelete(LanguageContext* pToAdd)
{
	for( USHORT i = 0; i < m_aSwitchCtxDelete.Count(); ++i )
	{
		LanguageContext* pCmp = m_aSwitchCtxDelete.GetObject(i);
		if( pCmp->nLanguage == pToAdd->nLanguage	&&
			pCmp->isProg	== pToAdd->isProg		&&
			pCmp->isDoc		== pToAdd->isDoc )
			return;
	}
	m_aSwitchCtxDelete.Insert(pToAdd,LIST_APPEND);
}

void SiEnvironment::SetArchive(ArchDirectory* pNew, FncFindArch fncFindArch)
{
	if( pArchive )
		delete pArchive;
	pArchive = pNew;
	pArchive->SetFindArchHdl( fncFindArch );
}

void SiEnvironment::InitStartPath()
{
	SiDirEntry anAppEntry(Application::GetAppFileName());
	anAppEntry.ToAbs();
	m_aStartPath = ((const SiDirEntry&)anAppEntry.GetPath()).GetFull();

	SiDirEntry aKHPatch( m_aStartPath );
	aKHPatch += ByteString(KH_PATCH_DETECTOR);
	if( aKHPatch.Exists() )
		m_bHasKHPatched = TRUE;
};

///////////////////////////////////////////////////////////////////////////////
//
//		Store Environment
//

#define MAIN_KEY	"HKEY_CURRENT_USER"
#define SUB_KEY		"Software\\Sun Microsystems\\setup\\recycle"

void SiEnvironment::StoreRecycle()
{
	#ifdef WNT
	WinOS::Register(MAIN_KEY, SUB_KEY, "version",			1, FALSE );

	WinOS::Register(MAIN_KEY, SUB_KEY, "lastname",			m_aUserName, FALSE );
	WinOS::Register(MAIN_KEY, SUB_KEY, "firstname",			m_aUserFirstName, FALSE );
	WinOS::Register(MAIN_KEY, SUB_KEY, "street",			m_aStreet, FALSE );
	WinOS::Register(MAIN_KEY, SUB_KEY, "zip",				m_aZip, FALSE );
	WinOS::Register(MAIN_KEY, SUB_KEY, "city",				m_aCity, FALSE );
	WinOS::Register(MAIN_KEY, SUB_KEY, "country",			m_aCountry, FALSE );
	WinOS::Register(MAIN_KEY, SUB_KEY, "countryid",			ByteString::CreateFromInt32(m_nCountry), FALSE );
	WinOS::Register(MAIN_KEY, SUB_KEY, "company",			m_aCompanyName, FALSE );
	WinOS::Register(MAIN_KEY, SUB_KEY, "customernr",		m_aCustomerNr, FALSE );
	WinOS::Register(MAIN_KEY, SUB_KEY, "id",				m_aUserId, FALSE );
	WinOS::Register(MAIN_KEY, SUB_KEY, "email",				m_aUserEmail, FALSE );
	#else
	#endif
}

void SiEnvironment::InitRecycle()
{
	#ifdef WNT
	USHORT nVersion			= (USHORT) ByteString(WinOS::GetRegValue(MAIN_KEY, SUB_KEY, "version")).ToInt32();

	m_aUserName				= WinOS::GetRegValue( MAIN_KEY, SUB_KEY, "lastname" );
	m_aUserFirstName		= WinOS::GetRegValue( MAIN_KEY, SUB_KEY, "firstname" );
	m_aStreet				= WinOS::GetRegValue( MAIN_KEY, SUB_KEY, "street" );
	m_aZip					= WinOS::GetRegValue( MAIN_KEY, SUB_KEY, "zip" );
	m_aCity					= WinOS::GetRegValue( MAIN_KEY, SUB_KEY, "city" );
	m_aCountry				= WinOS::GetRegValue( MAIN_KEY, SUB_KEY, "country" );
	m_nCountry				= (USHORT) ByteString(WinOS::GetRegValue(MAIN_KEY, SUB_KEY, "countryid")).ToInt32();
	m_aCompanyName			= WinOS::GetRegValue( MAIN_KEY, SUB_KEY, "company" );
	m_aCustomerNr			= WinOS::GetRegValue( MAIN_KEY, SUB_KEY, "customernr" );
	m_aUserId				= WinOS::GetRegValue( MAIN_KEY, SUB_KEY, "id" );
	m_aUserEmail 			= WinOS::GetRegValue( MAIN_KEY, SUB_KEY, "email" );

	#else
	#endif
}
