/*************************************************************************
 *
 *  $RCSfile: datasrc.cxx,v $
 *
 *  $Revision: 1.5 $
 *
 *  last change: $Author: ka $ $Date: 2001/07/30 15:55:39 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _COM_SUN_STAR_IO_XACTIVEDATASOURCE_HPP_
#include <com/sun/star/io/XActiveDataSource.hpp>
#endif
#ifndef _COM_SUN_STAR_IO_XACTIVEDATACONTROL_HPP_
#include <com/sun/star/io/XActiveDataControl.hpp>
#endif
#ifndef _COM_SUN_STAR_LANG_XINITIALIZATION_HPP_
#include <com/sun/star/lang/XInitialization.hpp>
#endif

#include <unotools/ucbstreamhelper.hxx>
#include <unotools/localfilehelper.hxx>
#include <cppuhelper/implbase3.hxx>
#ifndef _URLOBJ_HXX
#include <tools/urlobj.hxx>
#endif
#ifndef _STREAM_HXX
#include <tools/stream.hxx>
#endif

#include "datasrc.hxx"

using namespace ::com::sun::star;
using namespace ::rtl;

class FileSource_Impl : public ::cppu::WeakAggImplHelper3< io::XActiveDataSource, io::XActiveDataControl, lang::XInitialization >
{
private:
    SvStream*                               pStream;
    uno::Reference< io::XStreamListener >   m_xListener;
    uno::Reference< io::XOutputStream >     m_xSink;

public:
                    FileSource_Impl();
    virtual         ~FileSource_Impl();

                    // io::XActiveDataControl
    virtual void SAL_CALL addListener(const uno::Reference< io::XStreamListener > & aListener) throw( uno::RuntimeException );
    virtual void SAL_CALL removeListener(const uno::Reference< io::XStreamListener > & aListener) throw( uno::RuntimeException );
    virtual void SAL_CALL start(void) throw( uno::RuntimeException );
    virtual void SAL_CALL terminate(void) throw( uno::RuntimeException );

                    // io::XActiveDataSource
    virtual void SAL_CALL setOutputStream(const uno::Reference< io::XOutputStream > & aStream) throw( uno::RuntimeException );
    virtual uno::Reference< io::XOutputStream > SAL_CALL getOutputStream(void) throw( uno::RuntimeException );

                    // lang::XInitialization
    virtual void SAL_CALL initialize(const uno::Sequence< uno::Any >& Arguments) throw( uno::Exception, uno::RuntimeException );

};

FileSource_Impl::FileSource_Impl()
    : pStream( NULL )
{
}

FileSource_Impl::~FileSource_Impl()
{
    delete pStream;
}


void SAL_CALL FileSource_Impl::initialize( const uno::Sequence< uno::Any >& Arguments) throw( uno::Exception, uno::RuntimeException )
{
    const uno::Any *pArr = Arguments.getConstArray();
    OUString aName;
    pArr[0] >>= aName;
    if ( aName.getLength() )
    {
        INetURLObject aURL( aName );

        if( aURL.GetProtocol() == INET_PROT_NOT_VALID )
        {
            String aURLStr;

            ::utl::LocalFileHelper::ConvertPhysicalNameToURL( aName, aURLStr );
            aURL = INetURLObject( aURLStr );
        }

        if( aURL.GetProtocol() != INET_PROT_NOT_VALID )
            pStream = ::utl::UcbStreamHelper::CreateStream( aURL.GetMainURL( INetURLObject::NO_DECODE ), STREAM_STD_READ );
    }
}

void SAL_CALL FileSource_Impl::addListener(const uno::Reference< io::XStreamListener > & aListener) throw( uno::RuntimeException )
{
/*
    if( m_xListener.is() )
        fprintf( stderr, "Warning: addSourceControllerListener called when already having a listener\n" );
*/
    m_xListener = aListener;
}

void SAL_CALL FileSource_Impl::removeListener(const uno::Reference< io::XStreamListener > & aListener) throw( uno::RuntimeException )
{
    m_xListener = uno::Reference< io::XStreamListener > ();
}

void SAL_CALL FileSource_Impl::setOutputStream( const uno::Reference< io::XOutputStream > & Listener )
{
    m_xSink = Listener;
}

uno::Reference< io::XOutputStream > SAL_CALL FileSource_Impl::getOutputStream()
{
    return m_xSink;
}

void SAL_CALL FileSource_Impl::start() throw( uno::RuntimeException )
{
    if ( pStream && m_xSink.is() )
    {
        sal_Int8 *pBuf = new sal_Int8[ 65536 ];
        sal_uInt32 nBytes = 1;
        while( nBytes )
        {
            nBytes = pStream->Read( pBuf, 65536 );
            if ( nBytes )
                m_xSink->writeBytes( uno::Sequence<sal_Int8>( pBuf, nBytes ) );
        }

        delete pBuf;

        uno::Reference< io::XActiveDataSource >  xRef( this );
        m_xSink->closeOutput();
        m_xSink = uno::Reference< io::XOutputStream > ();
        if( m_xListener.is() )
            m_xListener->closed();
        m_xListener = uno::Reference< io::XStreamListener > ();
    }
}

void SAL_CALL FileSource_Impl::terminate() throw( uno::RuntimeException )
{
    uno::Reference< io::XActiveDataSource >  xRef( this );
    if ( m_xSink.is() )
        m_xSink->closeOutput();
    m_xSink = uno::Reference< io::XOutputStream > ();
    if( m_xListener.is() )
        m_xListener->closed();
    m_xListener = uno::Reference< io::XStreamListener > ();
}

///////////////////////////////////////////////////////////////////////

SimpleDataSourceFactory::SimpleDataSourceFactory()
{
}

SimpleDataSourceFactory::~SimpleDataSourceFactory()
{
}

uno::Reference< uno::XInterface > SAL_CALL SimpleDataSourceFactory::createInstance(const OUString& ServiceSpecifier) throw( uno::Exception, uno::RuntimeException )
{
    INetProtocol eProt = INetURLObject::CompareProtocolScheme( ServiceSpecifier );
    switch ( eProt )
    {
        case INET_PROT_FILE :
            break;
        default:
            return uno::Reference< uno::XInterface > ();
            break;
    }

    return (::cppu::OWeakObject*) new FileSource_Impl;
}

uno::Reference< uno::XInterface > SAL_CALL SimpleDataSourceFactory::createInstanceWithArguments(const OUString& ServiceSpecifier, const uno::Sequence< uno::Any >& Arguments) throw( uno::Exception, uno::RuntimeException )
{
    INetProtocol eProt = INetURLObject::CompareProtocolScheme( ServiceSpecifier );
    switch ( eProt )
    {
        case INET_PROT_FILE :
            break;
        default:
            return uno::Reference< uno::XInterface > ();
            break;
    }

    uno::Reference< uno::XInterface >  xRet;
    FileSource_Impl* pSource = new FileSource_Impl();
    xRet = (::cppu::OWeakObject*) pSource;
    pSource->initialize( Arguments );
    return xRet;
}

uno::Sequence< OUString > SAL_CALL SimpleDataSourceFactory::getAvailableServiceNames(void) throw( uno::RuntimeException )
{
    uno::Sequence< OUString > aRet(1);
    OUString *pArr = aRet.getArray();
    pArr[0] = rtl::OUString::createFromAscii("file:");

    return aRet;
}

uno::Reference< uno::XInterface > SimpleDataSourceFactory_CreateInstance( const uno::Reference< lang::XMultiServiceFactory >  & rSMgr )
{
    return *( new SimpleDataSourceFactory() );
}
