/*************************************************************************
 *
 *  $RCSfile: smanager.cxx,v $
 *
 *  $Revision: 1.4 $
 *
 *  last change: $Author: th $ $Date: 2001/05/11 09:48:06 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <tools/debug.hxx>

#include <hash_map>
#include <hash_set>

#include <rtl/uuid.h>
#include <rtl/memory.h>

#ifndef _COM_SUN_STAR_LANG_XSERVICENAME_HPP_
#include <com/sun/star/lang/XServiceName.hpp>
#endif
#ifndef _COM_SUN_STAR_LANG_XSERVICEINFO_HPP_
#include <com/sun/star/lang/XServiceInfo.hpp>
#endif
#ifndef _COM_SUN_STAR_LANG_XMULTISERVICEFACTORY_HPP_
#include <com/sun/star/lang/XMultiServiceFactory.hpp>
#endif
#ifndef _COM_SUN_STAR_LANG_XSINGLESERVICEFACTORY_HPP_
#include <com/sun/star/lang/XSingleServiceFactory.hpp>
#endif
#ifndef _COM_SUN_STAR_LANG_XEVENTLISTENER_HPP_
#include <com/sun/star/lang/XEventListener.hpp>
#endif
#ifndef _COM_SUN_STAR_CONTAINER_XSET_HPP_
#include <com/sun/star/container/XSet.hpp>
#endif
#ifndef _COM_SUN_STAR_CONTAINER_XCONTENTENUMERATIONACCESS_HPP_
#include <com/sun/star/container/XContentEnumerationAccess.hpp>
#endif

#ifndef _CPPUHELPER_COMPONENT_HXX_
#include <cppuhelper/component.hxx>
#endif
#ifndef _CPPUHELPER_WEAKREF_HXX_
#include <cppuhelper/weakref.hxx>
#endif

#include <cppuhelper/implbase1.hxx>

#ifndef _VCL_MSGBOX_HXX
#include <vcl/msgbox.hxx>
#endif

#include <vos/mutex.hxx>
#include <vos/diagnose.hxx>
#include <vos/dynload.hxx>
#include <vos/module.hxx>

#ifndef _RTL_USTRING_HXX_
#include <rtl/ustring.hxx>
#endif

using namespace ::osl;
using namespace ::vos;
using namespace ::rtl;
using namespace ::com::sun::star;

#ifdef DBG_UTIL
void REPORT_MISSING_SERVICE( const OUString& ServiceSpecifier )
{
    // check for optional services
    if( ServiceSpecifier.compareToAscii( "com.sun.star.frame.Settings" ) == 0 )
        return;

    // report a newly? missing service
    ByteString aErrorBox;
    aErrorBox += "StarOffice Player proccess service manager\nfailed to create the service:\n\n";
    aErrorBox += ByteString( UniString(ServiceSpecifier), RTL_TEXTENCODING_ASCII_US );
    aErrorBox += "\n\nPlease check if this service is needed for player operation";

    DBG_ERROR( aErrorBox.GetBuffer() );
}
#else
#define REPORT_MISSING_SERVICE(x)
#endif

/*****************************************************************************
    Enumeration by ServiceName
*****************************************************************************/
struct hashRef_Impl
{
    size_t operator()(const uno::Reference< uno::XInterface >  & rName) const
    {
        // query to uno::XInterface. The cast to uno::XInterface* must be the same for the same object
        uno::Reference< uno::XInterface >  x( rName, uno::UNO_QUERY );
        return (size_t)(uno::XInterface *)x.get();
    }
};

struct equaltoRef_Impl
{
    size_t operator()(const uno::Reference< uno::XInterface >  & rName1, const uno::Reference< uno::XInterface >  & rName2 ) const
    {
        return rName1 == rName2;
    }
};

typedef ::std::hash_set
<
    uno::Reference< uno::XInterface > ,
    hashRef_Impl,
    equaltoRef_Impl
> HashSet_Ref;

class ServiceEnumeration_Impl : public ::cppu::WeakImplHelper1< container::XEnumeration >
{
public:
            ServiceEnumeration_Impl( const uno::Sequence< uno::Reference< uno::XInterface >  > & rFactories )
                : aFactories( rFactories )
                , nIt( 0 )
            {
            }
            ~ServiceEnumeration_Impl() {};

    // container::XEnumeration
    sal_Bool SAL_CALL hasMoreElements(void) throw(  );
    uno::Any SAL_CALL nextElement(void) throw( container::NoSuchElementException  );

private:
    Mutex aMutex;
    uno::Sequence< uno::Reference< uno::XInterface > > aFactories;
    sal_Int32 nIt;
};

// container::XEnumeration
sal_Bool SAL_CALL ServiceEnumeration_Impl::hasMoreElements(void) throw(  )
{
    MutexGuard aGuard( aMutex );
    return nIt != aFactories.getLength();
}

// container::XEnumeration
uno::Any SAL_CALL ServiceEnumeration_Impl::nextElement(void) throw( container::NoSuchElementException  )
{
    MutexGuard aGuard( aMutex );
    if( nIt == aFactories.getLength() )
        throw container::NoSuchElementException();

    uno::Any ret;
    ret <<= (aFactories.getConstArray()[nIt++]);
    return ret;
}

/*****************************************************************************
    Enumeration by implementation
*****************************************************************************/
class ImplementationEnumeration_Impl : public ::cppu::WeakImplHelper1< container::XEnumeration >
{
public:
            ImplementationEnumeration_Impl( const HashSet_Ref & rImplementationMap )
                : aImplementationMap( rImplementationMap )
            {
                aIt = aImplementationMap.begin();
            }
            ~ImplementationEnumeration_Impl() {};

    // container::XEnumeration
    sal_Bool SAL_CALL hasMoreElements(void) throw(  );
    uno::Any SAL_CALL nextElement(void) throw( container::NoSuchElementException  );
private:
    Mutex                           aMutex;
    HashSet_Ref                     aImplementationMap;
    HashSet_Ref::iterator           aIt;
    sal_Int32                       nNext;
    uno::Reference< uno::XInterface > xNext;
};

// container::XEnumeration
sal_Bool SAL_CALL ImplementationEnumeration_Impl::hasMoreElements(void) throw(  )
{
    MutexGuard aGuard( aMutex );
    return aIt != aImplementationMap.end();
}

// container::XEnumeration
uno::Any SAL_CALL ImplementationEnumeration_Impl::nextElement(void) throw( container::NoSuchElementException  )
{
    MutexGuard aGuard( aMutex );
    if( aIt == aImplementationMap.end() )
        throw container::NoSuchElementException();

    uno::Any ret;
    ret <<= *aIt;
    aIt++;
    return ret;
}

/*****************************************************************************
    Hash tables
*****************************************************************************/
struct equalUString_Impl
{
  bool operator()(const OUString & s1, const OUString & s2) const
  {
    return (s1 == s2) != 0;
  }
};

struct hashUString_Impl
{
    size_t operator()(const OUString & rName) const
    {
        return rName.hashCode();
    }
};

typedef ::std::hash_set
<
    OUString,
    hashUString_Impl,
    equalUString_Impl
> HashSet_UString;

typedef ::std::hash_multimap
<
    OUString,
    uno::Reference< uno::XInterface > ,
    hashUString_Impl,
    equalUString_Impl
> HashMultimap_UString_Interface;

typedef ::std::hash_map
<
    OUString,
    uno::Reference< uno::XInterface > ,
    hashUString_Impl,
    equalUString_Impl
> HashMap_UString_Interface;

/*****************************************************************************
    class OSipServiceManager_Listener
*****************************************************************************/
class OSipServiceManager_Listener : public ::cppu::WeakImplHelper1< lang::XEventListener >
{
private:
    uno::WeakReference<  container::XSet > xSMgr;
public:
                            OSipServiceManager_Listener( const uno::Reference< container::XSet >  & rSMgr )
                                : xSMgr( rSMgr )
                                {}
    // lang::XEventListener
    void SAL_CALL               disposing(const lang::EventObject & rEvt )
                                {
                                    uno::Reference< container::XSet > x( xSMgr );
                                    if( x.is() )
                                    {
                                        try
                                        {
                                            x->remove( uno::Any( &rEvt.Source, ::getCppuType((const uno::Reference< uno::XInterface >*)0) ) );
                                        }
                                        catch(...)
                                        {
                                            VOS_ENSHURE( sal_False, "ex eption in remove" );
                                        }
                                    }
                                }
};



/*****************************************************************************
    class OSipServiceManager
*****************************************************************************/
struct OSipServiceManagerMutex
{
    Mutex                           m_mutex;
};

class OSipServiceManager : public lang::XMultiServiceFactory
                      , public container::XSet
                      , public container::XContentEnumerationAccess
                      , public OSipServiceManagerMutex
                      , public ::cppu::OComponentHelper
{
public:
                                OSipServiceManager();
                                ~OSipServiceManager();

    uno::Any SAL_CALL queryInterface( const uno::Type & rType ) throw(uno::RuntimeException);
    void SAL_CALL acquire() { OComponentHelper::acquire(); }
    void SAL_CALL release() { OComponentHelper::release(); }

    uno::Sequence< sal_Int8 > SAL_CALL getImplementationId(void);

    // lang::XMultiServiceFactory
    uno::Sequence< OUString > SAL_CALL getAvailableServiceNames() throw(  );
    uno::Reference< uno::XInterface > SAL_CALL createInstance(const OUString &) throw( uno::Exception );
    uno::Reference< uno::XInterface > SAL_CALL createInstanceWithArguments(const OUString &, const uno::Sequence<uno::Any>& Arguments) throw( uno::Exception );

    // The same as the interface method, but only uique names
    uno::Sequence< OUString > getAvailableServiceNames( HashSet_UString & aNameSet ) throw(  );

    // container::XElementAccess
    uno::Type SAL_CALL getElementType(void) throw(  );
    sal_Bool SAL_CALL hasElements(void) throw(  );

    // container::XEnumerationAccess
    uno::Reference< container::XEnumeration > SAL_CALL createEnumeration();

    // container::XSet
    sal_Bool SAL_CALL has( const uno::Any & Element ) throw(  );
    void SAL_CALL insert( const uno::Any & Element ) throw( lang::IllegalArgumentException, container::ElementExistException  );
    void SAL_CALL remove( const uno::Any & Element ) throw( lang::IllegalArgumentException, container::NoSuchElementException  );

    // container::XContentEnumerationAccess
    //uno::Sequence< OUString >         getAvailableServiceNames() throw( uno::Exception );
    uno::Reference< container::XEnumeration > SAL_CALL createContentEnumeration(const OUString& aServiceName) throw(  );

    // lang::XComponent
    void SAL_CALL dispose();

protected:
    sal_Bool haveFactoryWithThisImplementation(const OUString& aImplName);

    virtual uno::Reference< lang::XSingleServiceFactory >   queryServiceFactory(const OUString& aServiceName);
private:

    uno::Reference< lang::XEventListener >          getFactoryListener();

    HashMultimap_UString_Interface  m_ServiceMap;
    HashSet_Ref                     m_ImplementationMap;
    HashMap_UString_Interface       m_ImplementationNameMap;
    uno::Reference< lang::XEventListener >              xFactoryListener;
};

/**
 * Create a lang::ServiceManager
 */
OSipServiceManager::OSipServiceManager()
    : OComponentHelper( m_mutex )
{
}

/**
 * Destroy the lang::ServiceManager
 */
OSipServiceManager::~OSipServiceManager()
{
}

uno::Sequence< sal_Int8 > SAL_CALL OSipServiceManager::getImplementationId(void)
{
    static uno::Sequence< sal_Int8 > aId;
    if( aId.getLength() == 0 )
    {
        aId.realloc( 16 );
        rtl_createUuid( (sal_uInt8 *)aId.getArray(), 0, sal_True );
    }
    return aId;
}

// OComponentHelper
void OSipServiceManager::dispose()
{
    // notify the disposing listener, the service manager is still full alive
    uno::Reference< uno::XInterface > xInt( (::cppu::OWeakObject*)this );
    lang::EventObject aEvt( xInt );
    rBHelper.aLC.disposeAndClear( aEvt );

    // dispose all factories
    HashSet_Ref aImpls;
    {
        MutexGuard aGuard( m_mutex );
        aImpls = m_ImplementationMap;
    }
    HashSet_Ref::iterator aIt = aImpls.begin();
    while( aIt != aImpls.end() )
    {
        uno::Reference< lang::XComponent >  xComp( (*aIt++), uno::UNO_QUERY );
        if( xComp.is() )
            xComp->dispose();
    }

    // set the service manager to disposed
    OComponentHelper::dispose();

    // dispose
    HashSet_Ref aImplMap;
    {
        MutexGuard aGuard( m_mutex );
        // erase all members
        m_ServiceMap = HashMultimap_UString_Interface();
        aImplMap = m_ImplementationMap;
        m_ImplementationMap = HashSet_Ref();
        m_ImplementationNameMap = HashMap_UString_Interface();
    }

    // not only the Event should hold the object
    VOS_ASSERT( m_refCount != 1 );
}

/**
 * Create the lang::ServiceManager
 */
uno::Reference< lang::XMultiServiceFactory >  OSipServiceManager_CreateInstance() throw( uno::Exception )
{
    return new OSipServiceManager();
}

// OSipServiceManager
uno::Reference< lang::XEventListener > OSipServiceManager::getFactoryListener()
{
    MutexGuard aGuard( m_mutex );
    if( !xFactoryListener.is() )
        xFactoryListener = new OSipServiceManager_Listener( this );
    return xFactoryListener;
}

// uno::XInterface
uno::Any SAL_CALL OSipServiceManager::queryInterface( const uno::Type & rType ) throw( uno::RuntimeException )
{
    uno::Any aAny;

    if( rType == ::getCppuType((const uno::Reference< lang::XMultiServiceFactory >*)0) )
        aAny <<= uno::Reference< lang::XMultiServiceFactory >(this);
    else if( rType == ::getCppuType((const uno::Reference< container::XContentEnumerationAccess >*)0) )
        aAny <<= uno::Reference< container::XContentEnumerationAccess>(this);
    else if( rType == ::getCppuType((const uno::Reference< container::XSet >*)0) )
        aAny <<= uno::Reference< container::XSet>(this);
    else if( rType == ::getCppuType((const uno::Reference< container::XEnumerationAccess >*)0) )
        aAny <<= uno::Reference< container::XEnumerationAccess>(this);
    else if( rType == ::getCppuType((const uno::Reference< container::XElementAccess >*)0) )
        aAny <<= uno::Reference< container::XElementAccess >(this);
    else
        return OComponentHelper::queryInterface( rType );

    return aAny;
}

// lang::XMultiServiceFactory, ucb::XContentEnumeration
uno::Sequence< OUString > OSipServiceManager::getAvailableServiceNames( HashSet_UString & aNameSet ) throw(  )
{
    MutexGuard aGuard( m_mutex );

    HashMultimap_UString_Interface::iterator aSIt = m_ServiceMap.begin();
    while( aSIt != m_ServiceMap.end() )
        aNameSet.insert( (*aSIt++).first );

    /* do not return the implementation names
    HashMap_UString_Interface       m_ImplementationNameMap;
    HashMap_UString_Interface::iterator aIt = m_ImplementationNameMap.begin();
    while( aIt != m_ImplementationNameMap.end() )
        aNameSet.insert( (*aIt++).first );
    */

    uno::Sequence< OUString > aNames( aNameSet.size() );
    OUString * pArray = aNames.getArray();
    sal_Int32 i = 0;
    HashSet_UString::iterator next = aNameSet.begin();
    while( next != aNameSet.end() )
        pArray[i++] = (*next++);

    return aNames;
}

// lang::XMultiServiceFactory, ucb::XContentEnumeration
uno::Sequence< OUString > SAL_CALL OSipServiceManager::getAvailableServiceNames() throw(  )
{
    // all names
    HashSet_UString aNameSet;
    return getAvailableServiceNames( aNameSet );
}

// XMultibleServiceFactory
uno::Reference< uno::XInterface > SAL_CALL OSipServiceManager::createInstance(const OUString& ServiceSpecifier ) throw( uno::Exception )
{
    ClearableMutexGuard aGuard( m_mutex );

    uno::Reference< lang::XSingleServiceFactory >  xFactory( queryServiceFactory( ServiceSpecifier ) );

    uno::Reference< uno::XInterface >  xRet;
    if( xFactory.is() )
    {
        aGuard.clear();
        xRet = xFactory->createInstance();
    }
    else
    {
        REPORT_MISSING_SERVICE( ServiceSpecifier );
    }
    return xRet;
}

// XMultibleServiceFactory
uno::Reference< uno::XInterface > SAL_CALL OSipServiceManager::createInstanceWithArguments
(
    const OUString& ServiceSpecifier,
    const uno::Sequence<uno::Any>& Arguments
) throw( uno::Exception )
{
    ClearableMutexGuard aGuard( m_mutex );
    uno::Reference< uno::XInterface >  xRet;
    uno::Reference< lang::XSingleServiceFactory >  xFactory( queryServiceFactory( ServiceSpecifier ) );
    aGuard.clear();

    if( xFactory.is() )
    {
        // single service factory found
        if( Arguments.getLength() )
            xRet = xFactory->createInstanceWithArguments( Arguments );
        else
            xRet = xFactory->createInstance();
    }
    else
    {
        REPORT_MISSING_SERVICE( ServiceSpecifier );
    }
    return xRet;
}

uno::Reference< lang::XSingleServiceFactory > OSipServiceManager::queryServiceFactory(const OUString& aServiceName)
{
    MutexGuard aGuard( &m_mutex );
    uno::Reference< lang::XSingleServiceFactory >  xRet;

    HashMultimap_UString_Interface::iterator aMultiIt =
        ((OSipServiceManager*)this)->m_ServiceMap.find( aServiceName );
    if( aMultiIt == ((OSipServiceManager*)this)->m_ServiceMap.end() )
    {
        // no service found, look for an implementation
        HashMap_UString_Interface::iterator aIt =
            ((OSipServiceManager*)this)->m_ImplementationNameMap.find( aServiceName );
        if( aIt != ((OSipServiceManager*)this)->m_ImplementationNameMap.end() )
        {
            // an implementation found
            uno::Any aAny( (*aIt).second->queryInterface( ::getCppuType(( const uno::Reference< lang::XSingleServiceFactory>*)0 )));
            aAny >>= xRet;
        }
    }
    else
    {
        (*aMultiIt).second->queryInterface( ::getCppuType((const uno::Reference< lang::XSingleServiceFactory >*)0) ) >>= xRet;
    }

    return xRet;
}

// container::XContentEnumerationAccess
uno::Reference< container::XEnumeration > SAL_CALL OSipServiceManager::createContentEnumeration(const OUString& aServiceName) throw(  )
{
    MutexGuard aGuard( ((OSipServiceManager *)this)->m_mutex );

    // get all factories
    ::std::pair<HashMultimap_UString_Interface::iterator, HashMultimap_UString_Interface::iterator>
        p = ((OSipServiceManager *)this)->m_ServiceMap.equal_range( aServiceName );

    if( p.first == p.second )
        // nothing
        return uno::Reference< container::XEnumeration > ();

    // get the length of the sequence
    sal_Int32 nLen = 0;
    HashMultimap_UString_Interface::iterator next = p.first;
    while( next != p.second )
    {
        nLen++;
        next++;
    }

    uno::Sequence< uno::Reference< uno::XInterface >  > aFactories( nLen );
    uno::Reference< uno::XInterface >  * pArray = aFactories.getArray();
    sal_Int32 i = 0;
    while( p.first != p.second )
    {
        pArray[i++] = (*p.first).second;
        p.first++;
    }
    return new ServiceEnumeration_Impl( aFactories );
}

// container::XEnumeration
uno::Reference< container::XEnumeration > SAL_CALL OSipServiceManager::createEnumeration()
{
    MutexGuard aGuard( m_mutex );
    return new ImplementationEnumeration_Impl( m_ImplementationMap );
}

// container::XElementAccess
uno::Type SAL_CALL OSipServiceManager::getElementType(void) throw(  )
{
    return ::getCppuType((const uno::Reference< uno::XInterface >*)0);
}

// container::XElementAccess
sal_Bool SAL_CALL OSipServiceManager::hasElements(void) throw(  )
{
    MutexGuard aGuard( &m_mutex );
    return !m_ImplementationMap.empty();
}

// container::XSet
sal_Bool SAL_CALL OSipServiceManager::has( const uno::Any & Element ) throw(  )
{
    if( Element.getValueTypeClass() == uno::TypeClass_INTERFACE )
    {
        MutexGuard aGuard( &m_mutex );
        // fix against uno::XInterface problems in the bridge
        uno::Reference< uno::XInterface >  xEle( *(uno::Reference< uno::XInterface > *)Element.getValue(), uno::UNO_QUERY );
        HashSet_Ref::iterator aIt =
            ((OSipServiceManager*)this)->m_ImplementationMap.find( xEle );
        return aIt != m_ImplementationMap.end();
    }
    return sal_False;
}

// container::XSet
void SAL_CALL OSipServiceManager::insert( const uno::Any & Element )
    throw( lang::IllegalArgumentException, container::ElementExistException )
{
    if( Element.getValueTypeClass() != uno::TypeClass_INTERFACE )
        throw lang::IllegalArgumentException();
    // fix against uno::XInterface problems in the bridge
    uno::Reference< uno::XInterface >  xEle( *(uno::Reference< uno::XInterface > *)Element.getValue(), uno::UNO_QUERY );
    if( !xEle.is() )
        throw lang::IllegalArgumentException();

    {
    MutexGuard aGuard( m_mutex );
    HashSet_Ref::iterator aIt = m_ImplementationMap.find( xEle );
    if( aIt != m_ImplementationMap.end() )
        throw container::ElementExistException();

    // put into the implementation hashmap
    m_ImplementationMap.insert( xEle );

    // put into the implementation name hashmap
    uno::Reference< lang::XServiceInfo >  xInfo( xEle, uno::UNO_QUERY );
    if( xInfo.is() )
    {
        OUString aImplName = xInfo->getImplementationName();
        if( aImplName.getLength() )
            m_ImplementationNameMap[ aImplName ] = xEle;
    }

    //put into the service map
    uno::Reference< lang::XServiceInfo >  xSF( xEle, uno::UNO_QUERY );
    if( xSF.is() )
    {
        uno::Sequence< OUString > aServiceNames = xSF->getSupportedServiceNames();
        const OUString * pArray = aServiceNames.getConstArray();
        for( sal_Int32 i = 0; i < aServiceNames.getLength(); i++ )
            m_ServiceMap.insert( HashMultimap_UString_Interface::value_type( pArray[i],
                                *(uno::Reference< uno::XInterface >  *)Element.getValue() ) );
    }
    }
    // add the disposing listener to the factory
    uno::Reference< lang::XComponent >  xComp( xEle, uno::UNO_QUERY );
    if( xComp.is() )
        xComp->addEventListener( getFactoryListener() );
}

// helper function
sal_Bool OSipServiceManager::haveFactoryWithThisImplementation(const OUString& aImplName)
{
    if( m_ImplementationNameMap.find(aImplName) != m_ImplementationNameMap.end())
        return sal_True;
    else
        return sal_False;
}

// container::XSet
void SAL_CALL OSipServiceManager::remove( const uno::Any & Element )
    throw( lang::IllegalArgumentException, container::NoSuchElementException )
{
    if( Element.getValueTypeClass() != uno::TypeClass_INTERFACE )
        throw lang::IllegalArgumentException();
    // fix against uno::XInterface problems in the bridge
    uno::Reference< uno::XInterface >  xEle( *(uno::Reference< uno::XInterface > *)Element.getValue(), uno::UNO_QUERY );
    if( !xEle.is() )
        throw lang::IllegalArgumentException();

    // remove the disposing listener from the factory
    uno::Reference< lang::XComponent >  xComp( xEle, uno::UNO_QUERY );
    if( xComp.is() )
        xComp->removeEventListener( getFactoryListener() );

    MutexGuard aGuard( m_mutex );
    HashSet_Ref::iterator aIt = m_ImplementationMap.find( xEle );
    if( aIt == m_ImplementationMap.end() )
        throw container::NoSuchElementException();

    // remove from the implementation map
    m_ImplementationMap.erase( aIt );
    // remove from the implementation name hashmap
    uno::Reference< lang::XServiceInfo >  xInfo( xEle, uno::UNO_QUERY );
    if( xInfo.is() )
    {
        OUString aImplName = xInfo->getImplementationName();
        if( aImplName.getLength() )
            m_ImplementationNameMap.erase( aImplName );
    }

    //remove from the service map
    uno::Reference< lang::XServiceInfo >  xSF( xEle, uno::UNO_QUERY );
    if( xSF.is() )
    {
        uno::Sequence< OUString > aServiceNames = xSF->getSupportedServiceNames();
        const OUString * pArray = aServiceNames.getConstArray();
        for( sal_Int32 i = 0; i < aServiceNames.getLength(); i++ )
        {
            ::std::pair<HashMultimap_UString_Interface::iterator, HashMultimap_UString_Interface::iterator> p =
                m_ServiceMap.equal_range( pArray[i] );

            while( p.first != p.second )
            {
                if( xEle == (*p.first).second )
                {
                    m_ServiceMap.erase( p.first );
                    break;
                }
                p.first++;
            }
        }
    }
}


extern "C"
{
typedef void (SAL_CALL *GetSmartFactory_Func)( const sal_Char*, const uno::Reference< lang::XMultiServiceFactory >  & rSMgr, uno::Reference< lang::XSingleServiceFactory > & xFactory );
}

// Load Dll
uno::Reference< lang::XSingleServiceFactory >  activateFromSharedLibrary
(
    const sal_Char * pImplementationName,
    const sal_Char * pDllName,
    const uno::Reference< lang::XMultiServiceFactory >  & xSMgr
)
{
    uno::Reference< lang::XSingleServiceFactory >  xRet;
#if 0

    const ::rtl::OUString aSrcDllName( ::rtl::OUString::createFromAscii( pDllName ) );
    ::rtl::OUString aDllName;

    ORealDynamicLoader::computeModuleName( aSrcDllName, aDllName );


    OModule* pModule = new OModule(aDllName);

    if( pModule->load( aDllName ) )
    {

        // try to get provider from module
        GetSmartFactory_Func pFactFunc = (GetSmartFactory_Func)pModule->getSymbol(::rtl::OUString(RTL_CONSTASCII_USTRINGPARAM("exService_getSmartFactory")));

        if (pFactFunc)
        {
            (*pFactFunc)(pImplementationName, xSMgr, xRet );
        }
    }

    if (!xRet.is() && pModule)
        delete pModule;
    else
    {
        // pModule will not be deleted anymore because there is no
        // way to find out at which time all object created in this
        // module are destroyed.
    }

#endif
    return xRet;
}

