/*************************************************************************
 *
 *  $RCSfile: options.cxx,v $
 *
 *  $Revision: 1.3 $
 *
 *  last change: $Author: thb $ $Date: 2001/06/05 11:55:30 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _TOOLS_DEBUG_HXX //autogen wg. DBG_ASSERT
#include <tools/debug.hxx>
#endif

#include <strings.hrc>
#include "options.hxx"


///////////////////////////////////////////////////////////////////////
// class Options
///////////////////////////////////////////////////////////////////////


// no need to cache anything, might even cause trouble when creating 
// second Options object for the presentation itself.
Options::Options(BOOL bPersistent) : 
		ConfigItem( OUString::createFromAscii( "Office.Player/Settings" ), CONFIG_MODE_IMMEDIATE_UPDATE ),
		m_bPersistent(bPersistent)
{
	Load();
	SetModified( FALSE );
}

Options::~Options()
{
	Commit();
}

void Options::Commit()
{
	if (IsModified())
		Save();
}

void Options::SetModified( BOOL bVal )
{
	if ((m_bModified = bVal))
	{
		ConfigItem::SetModified();
	}
}

Sequence< OUString > Options::GetPropertyNames() const
{
	static const char* aPropNames[] =
	{
			"PresentationChange",
			"PageTimeout",
			"Endless",
			"PauseTimeout",
			"ShowPauseLogo",
			"MouseVisible",
			"MouseAsPen",
			"AnimationsAllowed",
			"LockedPages",
			"AlwaysOnTop",
			"FullScreen",
			"PresentationFile"
	};

    const int nCount = sizeof( aPropNames ) / sizeof( aPropNames[0] );
	Sequence< OUString > aNames( nCount );
	OUString* pNames = aNames.getArray();
	for (INT32 i = 0;  i < nCount;  ++i)
		pNames[i] = OUString::createFromAscii( aPropNames[i] );

	return aNames;
}


BOOL Options::Load()
{
	BOOL bSucc = FALSE;
	
	Sequence< OUString > aNames = GetPropertyNames();
	INT32 nProps = aNames.getLength();
	
	const Sequence< Any > aValues = GetProperties( aNames );
	DBG_ASSERT( aValues.getLength() == aNames.getLength(), 
			"GetProperties failed" );
	
	if (nProps && aValues.getLength() == nProps)
	{
		bSucc = TRUE;
		
		const Any* pValues = aValues.getConstArray();
		for (USHORT i = 0;  i < nProps;  ++i)
		{
			const Any &rVal = pValues[i];
			sal_Int32 nTemp;	
			if (rVal.hasValue())
			{
			switch(i)
			{
				case 0:
					rVal >>= m_nPresChange;
					break;

				case 1:
					rVal >>= m_nPageTimeout;
					break;

				case 2:
					rVal >>= nTemp; m_eEndless = static_cast<TriState>(nTemp);
					break;

				case 3:
					rVal >>= m_nPauseTimeout;
					break;

				case 4:
					rVal >>= nTemp; m_eShowPauseLogo = static_cast<TriState>(nTemp);
					break;

				case 5:
					rVal >>= nTemp; m_eMouseVisible = static_cast<TriState>(nTemp);
					break;

				case 6:
					rVal >>= nTemp; m_eMouseAsPen = static_cast<TriState>(nTemp);
					break;

				case 7:
					rVal >>= nTemp; m_eAnimationAllowed = static_cast<TriState>(nTemp);
					break;

				case 8:
					rVal >>= nTemp; m_eLockedPages = static_cast<TriState>(nTemp);
					break;

				case 9:
					rVal >>= nTemp; m_eAlwaysOnTop = static_cast<TriState>(nTemp);
					break;

				case 10:
					rVal >>= nTemp; m_eFullScreen = static_cast<TriState>(nTemp);
					break;

				case 11:
					rVal >>= m_aPresFile;
					break;

				default:
					DBG_ERROR( "unexpected type" );
					bSucc = FALSE;
					break;	
			}
			}
			else
			{
				DBG_ERROR( "value missing" );
				bSucc = FALSE;
			}
		}
	}
	DBG_ASSERT( bSucc, "LoadConfig failed" );
	
	return bSucc;
}


BOOL Options::Save()
{
	if( m_bPersistent )	
	{
		BOOL bSucc = FALSE;
	
		const Sequence< OUString > aNames = GetPropertyNames();
		INT32 nProps = aNames.getLength();
		sal_Int32	nTemp;
		
		Sequence< Any > aValues( nProps );
		Any *pValue = aValues.getArray();
		
		pValue[0] <<= m_nPresChange;
		pValue[1] <<= m_nPageTimeout;
		nTemp = static_cast<sal_Int32>(m_eEndless); pValue[2] <<= nTemp;
		pValue[3] <<= m_nPauseTimeout;
		nTemp = static_cast<sal_Int32>(m_eShowPauseLogo); pValue[4] <<= nTemp;
		nTemp = static_cast<sal_Int32>(m_eMouseVisible); pValue[5] <<= nTemp;
		nTemp = static_cast<sal_Int32>(m_eMouseAsPen); pValue[6] <<= nTemp;
		nTemp = static_cast<sal_Int32>(m_eAnimationAllowed); pValue[7] <<= nTemp;
		nTemp = static_cast<sal_Int32>(m_eLockedPages); pValue[8] <<= nTemp;
		nTemp = static_cast<sal_Int32>(m_eAlwaysOnTop); pValue[9] <<= nTemp;
		nTemp = static_cast<sal_Int32>(m_eFullScreen); pValue[10] <<= nTemp;
		pValue[11] <<= m_aPresFile;
		
		bSucc |= PutProperties( aNames, aValues );
		
		if (bSucc)
			SetModified( FALSE );
		
		return bSucc;
	}
	else
		return TRUE;	
}
