/*************************************************************************
 *
 *  $RCSfile: pastedlg.cxx,v $
 *
 *  $Revision: 1.7 $
 *
 *  last change: $Author: jl $ $Date: 2001/07/31 08:41:42 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifdef WNT
#include <tools/prewin.h>
#include <windows.h>
#include <ole2.h>
#include <tools/postwin.h>
#endif

#include <pastedlg.hxx>
#include <svtools/svmedit.hxx>
#include <svtools/transfer.hxx>
#include <vcl/dialog.hxx>
#include <vcl/button.hxx>
#include <vcl/fixed.hxx>
#include <vcl/group.hxx>
#include <vcl/lstbox.hxx>
#include <vcl/msgbox.hxx>
#include <vcl/exchange.hxx>
#include <svuidlg.hxx>
#include <svuidlg.hrc>
#include <embobj.hxx>
#include <sot/formats.hxx>

#pragma hdrstop

#include "helpid.hrc"

class MdPasteObject;

/********************** SvPasteDlg ****************************************
**************************************************************************/
class SvPasteDlg : public MdPasteObject
{
public:
	SvPasteDlg( Window * pParent );

	ListBox&		ObjectLB()		{ return aLbInsertList; }
	FixedText&		ObjectSource()	{ return aFtObjectSource; }
	RadioButton&	PasteLink()		{ return aRbPasteLink; }
	CheckBox&		AsIconBox()		{ return aCbDisplayAsIcon; }
//	FixedIcon&		IconBox()		{ return aFiDisplay; }

	const String& 	GetObjString()	{ return aSObject; }
	void			SelectObject();

private:
	DECL_LINK( SelectHdl, ListBox * );
	DECL_LINK( DoubleClickHdl, ListBox * );
};

SvPasteDlg::SvPasteDlg( Window* pParent ) :

	MdPasteObject( pParent, ResId( MD_PASTE_OBJECT, SOAPP->GetResMgr() ) )
{
	SetHelpId( HID_PASTE_DLG );
	SetUniqueId( HID_PASTE_DLG );

	Font aFont = aFtObjectSource.GetFont();
	aFont.SetWeight( WEIGHT_LIGHT );
	aFtObjectSource.SetFont( aFont );
	aFtResult.SetFont( aFont );
	aOKButton1.Disable();

	ObjectLB().SetSelectHdl( LINK( this, SvPasteDlg, SelectHdl ) );
	ObjectLB().SetDoubleClickHdl( LINK( this, SvPasteDlg, DoubleClickHdl ) );
}

void SvPasteDlg::SelectObject()
{
	if ( aLbInsertList.GetEntryCount() &&
		 !aRbPaste.IsVisible() && !aRbPasteLink.IsVisible() )
	{
		aLbInsertList.SelectEntryPos(0);
		SelectHdl( &aLbInsertList );
	}
}

IMPL_LINK( SvPasteDlg, SelectHdl, ListBox *, pListBox )
{
	if ( !aOKButton1.IsEnabled() )
		aOKButton1.Enable();
	aFbResult.Show();
	String sStr( ResId( STR_PASTE, SOAPP->GetResMgr() ) );
	sStr.SearchAndReplace( String::CreateFromAscii( RTL_CONSTASCII_STRINGPARAM( "%1" ) ), ObjectLB().GetSelectEntry() );
	aFtResult.SetText( sStr );
	return 0;
}

IMPL_LINK_INLINE_START( SvPasteDlg, DoubleClickHdl, ListBox *, pListBox )
{
	EndDialog( RET_OK );
	return 0;
}
IMPL_LINK_INLINE_END( SvPasteDlg, DoubleClickHdl, ListBox *, pListBox )

/********************** SvPasteObjectDialog ******************************
**************************************************************************/
/*************************************************************************
|*    SvPasteObjectDialog::SvPasteObjectDialog()
|*
|*    Beschreibung
|*    Ersterstellung    MM 14.06.94
|*    Letzte Aenderung  MM 14.06.94
*************************************************************************/
void SvPasteObjectDialog::SetDefault()
{
	bLink   = FALSE;
	nAspect = ASPECT_CONTENT;
}

SvPasteObjectDialog::SvPasteObjectDialog()
{
	SetDefault();
}
SvPasteObjectDialog::~SvPasteObjectDialog()
{
	void * pStr = aSupplementTable.First();
	while( pStr )
	{
		delete (String *)pStr;
		pStr = aSupplementTable.Next();
	}
}

/*************************************************************************
|*    SvPasteObjectDialog::Insert()
|*
|*    Beschreibung
|*    Ersterstellung    MM 14.06.94
|*    Letzte Aenderung  KA 16.03.2001
*************************************************************************/
void SvPasteObjectDialog::Insert( SotFormatStringId nFormat, const String& rFormatName )
{
	String * pStr = new String( rFormatName );
	if( !aSupplementTable.Insert( nFormat, pStr ) )
		delete pStr;
}

// -----------------------------------------------------------------------------


/*************************************************************************
|*    SvPasteObjectDialog::Execute()
|*
|*    Beschreibung
|*    Ersterstellung    MM 14.06.94
|*    Letzte Aenderung  KA 16.03.2001
*************************************************************************/

ULONG SvPasteObjectDialog::Execute( Window* pParent,
		const ::com::sun::star::uno::Reference<
		::com::sun::star::datatransfer::XTransferable >& rxTransferable )
{
	TransferableDataHelper aHelper( rxTransferable );
	TransferableObjectDescriptor aDesc;

    if( aHelper.HasFormat( SOT_FORMATSTR_ID_OBJECTDESCRIPTOR ) )
	    aHelper.GetTransferableObjectDescriptor( SOT_FORMATSTR_ID_OBJECTDESCRIPTOR, aDesc );

	return Execute( pParent, aHelper.GetDataFlavorExVector(), aDesc );
}

ULONG SvPasteObjectDialog::Execute( Window* pParent,
								const DataFlavorExVector& rFormats,
								const TransferableObjectDescriptor& rDesc )
{
	//Dialogbox erzeugen und fuellen
	SvPasteDlg*	pDlg = new SvPasteDlg( pParent );
	String aSourceName, aTypeName;
	ULONG nSelFormat = 0;
	SvGlobalName aEmptyNm;

	pDlg->ObjectLB().SetUpdateMode( FALSE );

	DataFlavorExVector::iterator aIter( ((DataFlavorExVector&)rFormats).begin() ),
								 aEnd( ((DataFlavorExVector&)rFormats).end() );
	while( aIter != aEnd )
	{
		::com::sun::star::datatransfer::DataFlavor aFlavor( *aIter );
		SotFormatStringId nFormat = (*aIter++).mnSotId;

		String*	pName = (String*) aSupplementTable.Get( nFormat );
		String aName;

#ifdef WNT
/*
		if( !pName &&
			( nFormat == SOT_FORMATSTR_ID_EMBED_SOURCE_OLE || nFormat == SOT_FORMATSTR_ID_EMBEDDED_OBJ_OLE ) )
		{
			BOOL IsClipboardObject_Impl( SotDataObject * );
			if( IsClipboardObject_Impl( pDataObj ) )
			{
				IDataObject * pDO = NULL;
				OleGetClipboard( &pDO );
				if( pDO )
				{
					FORMATETC fe;
					STGMEDIUM stm;
					(fe).cfFormat=RegisterClipboardFormat( "Object Descriptor" );
					(fe).dwAspect=DVASPECT_CONTENT;
					(fe).ptd=NULL;
					(fe).tymed=TYMED_HGLOBAL;
					(fe).lindex=-1;

					if (SUCCEEDED(pDO->GetData(&fe, &stm)))
					{
						LPOBJECTDESCRIPTOR pOD=(LPOBJECTDESCRIPTOR)GlobalLock(stm.hGlobal);
						if( pOD->dwFullUserTypeName )
						{
							OLECHAR * pN = (OLECHAR *)(((BYTE *)pOD) + pOD->dwFullUserTypeName);
							aName.Append( pN );
							pName = &aName;
							// set format to ole object
							nFormat = SOT_FORMATSTR_ID_EMBED_SOURCE_OLE;
						}
						if( pOD->dwSrcOfCopy )
						{
							OLECHAR * pN = (OLECHAR *)(((BYTE *)pOD) + pOD->dwSrcOfCopy);
							aSourceName.Append( *pN++ );
						}
						else
							aSourceName = String( ResId( STR_UNKNOWN_SOURCE, SOAPP->GetResMgr() ) );
						GlobalUnlock(stm.hGlobal);
						ReleaseStgMedium(&stm);
					}
				}
			}
		}
*/
#endif

		if( pName )
		{
			aName = *pName;

			if( SOT_FORMATSTR_ID_EMBED_SOURCE == nFormat )
			{
				if( rDesc.maClassName != aEmptyNm )
				{
					aSourceName = rDesc.maDisplayName;

					if( rDesc.maClassName == aObjClassName )
						aName = aObjName;
					else
						aName = aTypeName = rDesc.maTypeName;
				}
			}
			else if( SOT_FORMATSTR_ID_LINK_SOURCE == nFormat )
			{
				pDlg->PasteLink().Enable();
				continue;
			}
			else if( !aName.Len() )
				aName = aFlavor.HumanPresentableName;

			if( LISTBOX_ENTRY_NOTFOUND == pDlg->ObjectLB().GetEntryPos( aName ) )
				pDlg->ObjectLB().SetEntryData(
					pDlg->ObjectLB().InsertEntry( aName ), (void*) nFormat );
		}
	}

	if( !aTypeName.Len() && !aSourceName.Len() )
	{
		if( rDesc.maClassName != aEmptyNm )
		{
			aSourceName = rDesc.maDisplayName;
			aTypeName = rDesc.maTypeName;
		}

		if( !aSourceName.Len() )
			aSourceName = String( ResId( STR_UNKNOWN_SOURCE,
											SOAPP->GetResMgr() ) );
	}

	pDlg->ObjectLB().SetUpdateMode( TRUE );
	pDlg->SelectObject();

	if( aSourceName.Len() )
	{
		if( aTypeName.Len() )
			aTypeName += '\n';

		aTypeName += aSourceName;
		aTypeName.ConvertLineEnd();
	}

	pDlg->ObjectSource().SetText( aTypeName );

	SetDefault();

	if( pDlg->Execute() )
	{
		bLink = pDlg->PasteLink().IsChecked();

		if( pDlg->AsIconBox().IsChecked() )
			nAspect = ASPECT_ICON;

		nSelFormat  = (ULONG)pDlg->ObjectLB().GetEntryData( pDlg->ObjectLB().GetSelectEntryPos() );
	}

	delete pDlg;

	return nSelFormat;
}

// -----------------------------------------------------------------------------

