/*************************************************************************
 *
 *  $RCSfile: multipat.cxx,v $
 *
 *  $Revision: 1.3 $
 *
 *  last change: $Author: pb $ $Date: 2001/06/22 07:36:45 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

// include ---------------------------------------------------------------

#ifndef _TOOLS_DEBUG_HXX
#include <tools/debug.hxx>
#endif
#ifndef _URLOBJ_HXX
#include <tools/urlobj.hxx>
#endif
#ifndef _SV_MSGBOX_HXX //autogen
#include <vcl/msgbox.hxx>
#endif
#ifndef _FILEDLGHELPER_HXX
#include <sfx2/filedlghelper.hxx>
#endif
#pragma hdrstop

#include "multipat.hxx"
#include "dialmgr.hxx"

#include "multipat.hrc"
#include "dialogs.hrc"

#ifndef _COMPHELPER_PROCESSFACTORY_HXX_
#include <comphelper/processfactory.hxx>
#endif

#ifndef  _COM_SUN_STAR_LANG_XMULTISERVICEFACTORY_HPP_
#include <com/sun/star/lang/XMultiServiceFactory.hpp>
#endif

#ifndef  _COM_SUN_STAR_UI_DIALOGS_XFOLDERPICKER_HPP_
#include <com/sun/star/ui/dialogs/XFolderPicker.hpp>
#endif
#ifndef  _COM_SUN_STAR_UI_DIALOGS_EXECUTABLEDIALOGRESULTS_HPP_
#include <com/sun/star/ui/dialogs/ExecutableDialogResults.hpp>
#endif

using namespace ::com::sun::star::lang;
using namespace ::com::sun::star::ui::dialogs;
using namespace ::com::sun::star::uno;

#define MULTIPATH_DELIMITER		';'

// struct MultiPath_Impl -------------------------------------------------

struct MultiPath_Impl
{
	BOOL	bEmptyAllowed;
	BOOL	bShowPathFiles;

	MultiPath_Impl( BOOL bAllowed ) : bEmptyAllowed( bAllowed ), bShowPathFiles( FALSE ) {}
};

// class SvxMultiPathDialog ----------------------------------------------

IMPL_LINK_INLINE_START( SvxMultiPathDialog, SelectHdl_Impl, ListBox *, pBox )
{
	BOOL bEnable = ( pImpl->bEmptyAllowed || pBox->GetEntryCount() > 1 );
	aDelBtn.Enable( bEnable && pBox->GetSelectEntryPos() != LISTBOX_ENTRY_NOTFOUND );
	return 0;
}
IMPL_LINK_INLINE_END( SvxMultiPathDialog, SelectHdl_Impl, ListBox *, pBox )

// -----------------------------------------------------------------------

IMPL_LINK( SvxMultiPathDialog, AddHdl_Impl, PushButton *, EMPTYARG )
{
    rtl::OUString aService( RTL_CONSTASCII_USTRINGPARAM( FOLDER_PICKER_SERVICE_NAME ) );
    Reference < XMultiServiceFactory > xFactory( ::comphelper::getProcessServiceFactory() );
	Reference < XFolderPicker > xFolderPicker( xFactory->createInstance( aService ), UNO_QUERY );

	if ( xFolderPicker->execute() == ExecutableDialogResults::OK )
	{
		INetURLObject aPath( xFolderPicker->getDirectory() );
		aPath.removeFinalSlash();
		String sInsPath = aPath.getFSysPath( INetURLObject::FSYS_DETECT );

		if ( LISTBOX_ENTRY_NOTFOUND != aPathLB.GetEntryPos( sInsPath ) )
		{
			String sMsg( SVX_RES( RID_MULTIPATH_DBL_ERR ) );
			sMsg.SearchAndReplaceAscii( "%1", sInsPath );
			InfoBox( this, sMsg ).Execute();
		}
		else
		{
			USHORT nPos = aPathLB.InsertEntry( sInsPath, LISTBOX_APPEND );
			aPathLB.SetEntryData( nPos, (void*)new String( sInsPath ) );
		}
		SelectHdl_Impl( &aPathLB );
	}
	return 0;
}

// -----------------------------------------------------------------------

IMPL_LINK( SvxMultiPathDialog, DelHdl_Impl, PushButton *, EMPTYARG )
{
	USHORT nPos = aPathLB.GetSelectEntryPos();
	aPathLB.RemoveEntry( nPos );
	USHORT nCnt = aPathLB.GetEntryCount();

	if ( nCnt )
	{
		nCnt--;

		if ( nPos > nCnt )
			nPos = nCnt;
		aPathLB.SelectEntryPos( nPos );
	}
	SelectHdl_Impl( &aPathLB );
	return 0;
}

// -----------------------------------------------------------------------

SvxMultiPathDialog::SvxMultiPathDialog( Window* pParent, BOOL bEmptyAllowed ) :

	ModalDialog( pParent, SVX_RES( RID_SVXDLG_MULTIPATH ) ),

	aPathLB		( this, ResId( LB_MULTIPATH ) ),
	aAddBtn		( this, ResId( BTN_ADD_MULTIPATH ) ),
	aDelBtn		( this, ResId( BTN_DEL_MULTIPATH ) ),
	aPathGB		( this, ResId( GB_MULTIPATH) ),
	aOKBtn		( this, ResId( BTN_MULTIPATH_OK ) ),
	aCancelBtn	( this, ResId( BTN_MULTIPATH_CANCEL ) ),
	aHelpButton	( this, ResId( BTN_MULTIPATH_HELP ) ),
	pImpl		( new MultiPath_Impl( bEmptyAllowed ) )

{
	FreeResource();
	aPathLB.SetSelectHdl( LINK( this, SvxMultiPathDialog, SelectHdl_Impl ) );
	aAddBtn.SetClickHdl( LINK( this, SvxMultiPathDialog, AddHdl_Impl ) );
	aDelBtn.SetClickHdl( LINK( this, SvxMultiPathDialog, DelHdl_Impl ) );
	SelectHdl_Impl( &aPathLB );
}

// -----------------------------------------------------------------------

SvxMultiPathDialog::~SvxMultiPathDialog()
{
	USHORT nPos = aPathLB.GetEntryCount();
	while(nPos--)
		delete (String*)aPathLB.GetEntryData(nPos);
	delete pImpl;
}

// -----------------------------------------------------------------------

String SvxMultiPathDialog::GetPath() const
{
	String aPath;
	char cDelim = MULTIPATH_DELIMITER;

	for ( USHORT i = 0; i < aPathLB.GetEntryCount(); ++i )
	{
		aPath += *(String*)aPathLB.GetEntryData(i);
		aPath += cDelim;
	}
	aPath.EraseTrailingChars( cDelim );
	return aPath;
}

// -----------------------------------------------------------------------

void SvxMultiPathDialog::SetPath( const String& rPath )
{
	char cDelim = MULTIPATH_DELIMITER;
	USHORT nPos;

	for ( USHORT i = 0; i < rPath.GetTokenCount( cDelim ); ++i )
	{
		nPos = aPathLB.InsertEntry( rPath.GetToken( i, cDelim ), LISTBOX_APPEND );
		aPathLB.SetEntryData( nPos, (void*)new String( rPath.GetToken( i, cDelim ) ) );

	}
	SelectHdl_Impl( &aPathLB );
}

// -----------------------------------------------------------------------

void SvxMultiPathDialog::ShowPathFiles()
{
	DBG_ERRORFILE( "not implemented any longer" );
}

// -----------------------------------------------------------------------

String SvxMultiPathDialog::CreateUIName( const String& rPath )
{
	if ( rPath.Len() < MAX_PATHLEN )
		return rPath;
	sal_Unicode cDelim = MULTIPATH_DELIMITER;
	USHORT nCount = rPath.GetTokenCount( cDelim );

	if ( 1 == nCount )
		return rPath;
	else
	{
		const sal_Unicode* pc = rPath.GetBuffer();
		const sal_Unicode* pEnd = pc;
		pEnd += rPath.Len();

		while ( pEnd != pc )
		{
			while ( *pEnd != cDelim && pEnd != pc )
				pEnd--;

			if ( *pEnd == cDelim )
			{
				USHORT nPos = (USHORT)( pEnd - pc );

				if ( nPos <= MAX_PATHLEN - 4 )
				{
					String aTemp = rPath.Copy( 0, nPos + 1 );
					aTemp += '.';
					aTemp += '.';
					aTemp += '.';
					return aTemp;
				}
				pEnd--;
			}
		}

		return rPath.GetToken( 0, cDelim );
	}
}

