#!/bin/sh

# check for command line parameters
if [ $# -lt 3 -o $# -gt 4 ]; then
  echo "Usage: $0 <office_inst_path> <singleproductname> <productversion> <default_language>"
  echo "Usage: $0 -remove <singleproductname> <productversion>"
  exit 1
fi

# remove all files of this product in types
if [ -d $HOME/.dt/types ]; then
  rm -f "$HOME/.dt/types/$2_$3_*" 2>/dev/null
  
  # remove vnd. files that contain product name
  for SOURCE in `grep -l "$2_$3" $HOME/.dt/types/vnd.* 2>/dev/null`; do
    rm -f $SOURCE
  done  
fi

# remove all files of this product in icons
if [ -d $HOME/.dt/icons ]; then
  rm -f "$HOME/.dt/icons/$2_$3_*" 2>/dev/null
fi

# check if only remove should be done
if [ "$1" = "-remove" ]; then
    exit 0
fi

# get locale from environment
if [ $LC_ALL ]; then 
  LOCALE=$LC_ALL
elif [ $LC_CTYPE ]; then 
  LOCALE=$LC_CTYPE
elif [ $LANG ]; then 
  LOCALE=$LANG
else
  LOCALE="C"
fi

if [ "$LOCALE" = "C" ]; then
    # forth parameter should be the default locale of the office installation
  echo "$0: locale not set - defaulting to $4."
  LOCALE=$4
elif [ ! -d "$1/share/cde/types/$LOCALE" ]; then 
  CHARSET=`echo $LOCALE | sed "s/.*\././"`

  # extract charset  
  if [ "$LOCALE" = "$CHARSET" ]; then
    CHARSET=""
  fi

  # extract language
  LANGUAGE=`echo $LOCALE | sed "s/_.*//"` 
  
  if [ -d "$1/share/cde/types/$LANGUAGE$CHARSET" ]; then
#    echo "$0: files for locale $LOCALE not installed - defaulting to $LANGUAGE$CHARSET."
    LOCALE=$LANGUAGE$CHARSET
  elif [ -d "$1/share/cde/types/$LANGUAGE" ]; then
#    echo "$0: files for locale $LOCALE not installed - defaulting to $LANGUAGE."
    LOCALE=$LANGUAGE
  else
#    echo "$0: files for locale $LOCALE not installed - defaulting to $4."
    LOCALE=$4
  fi
fi

# check if a least this directory exists
if [ ! -d "$1/share/cde/types/$LOCALE" ]; then 
  echo "$0: installation error: $1/share/cde/types/$LOCALE not found."
  exit 1
fi

# create $HOME/.dt if necessary
if [ ! -d $HOME/.dt ]; then
  mkdir $HOME/.dt;
fi

# create $HOME/.dt/types if necessary
if [ ! -d $HOME/.dt/types ]; then
  mkdir $HOME/.dt/types;
fi

# create $HOME/.dt/icons if necessary
if [ ! -d $HOME/.dt/icons ]; then
  mkdir $HOME/.dt/icons;
fi

# copy language specific types files
for SOURCE in `ls "$1/share/cde/types/$LOCALE"`; do
  if [ `echo $SOURCE | grep vnd` ]; then
    cp "$1/share/cde/types/$LOCALE/$SOURCE" $HOME/.dt/types/
  else
    cp "$1/share/cde/types/$LOCALE/$SOURCE" "$HOME/.dt/types/$2_$3_$SOURCE"
  fi
done

# copy icons files
for SOURCE in `ls "$1/share/cde/icons"`; do
  ICON=`echo $SOURCE | sed "s/.xpm/.pm/"`
  cp "$1/share/cde/icons/$SOURCE" "$HOME/.dt/icons/$2_$3_$ICON"
done

# check if cde currently running
CDE_RUNNING=`xlsatoms | grep _DT_WM_READY`
if [ -n "$CDE_RUNNING" ]; then
    dtaction ReloadActions 2>&1 >/dev/null;
fi
