%{
/*
 * lexer for parsing xml-property source files (*.xml)
 *
 */


/* enlarge token buffer to tokenize whole strings */
#undef YYLMAX
#define YYLMAX 64000

/* to enable debug output define LEXDEBUG */
#define LEXDEBUG		1
#ifdef LEXDEBUG
#define OUTPUT	fprintf
#else
#define OUTPUT(Par1,Par2);
#endif

/* table of possible token ids */
#include "tokens.h"
#include <stdlib.h>
#include <stdio.h>

/* external functions (C++ code, declared as extren "C" */
extern WorkOnTokenSet( int, char* );
extern Argument( char * );
extern InitXmlExport( char * );
extern EndXmlExport();
extern GetError();
extern SetError();
extern int IsXGF();
extern char *GetOutputFile( int argc, char* argv[]);
extern FILE *GetXmlFile();

/* forwards */
void YYWarning();

int bText=0;
int bGif=0;
%}

%p 24000
%e 1200
%n 500

%%

"<ResourceBundle"[ \t]+"package"[ \t]*=[^\<]+\>	{
	if ( bGif )
		WorkOnTokenSet( UNKNOWNCHAR, yytext );
	else
		WorkOnTokenSet( XML_RESSOURCEBUNDLE_START, yytext );
}

"<OutputDir"[ \t]+"path"[ \t]*=[^\<]+\>	{
	if ( !bGif )
		WorkOnTokenSet( UNKNOWNCHAR, yytext );
	else
		WorkOnTokenSet( XML_RESSOURCEBUNDLE_START, yytext );
}

"</ResourceBundle>"	{
	if ( bGif )
		WorkOnTokenSet( UNKNOWNCHAR, yytext );
	else
		WorkOnTokenSet( XML_RESSOURCEBUNDLE_END, yytext );
}

"</OutputDir>"	{
	if ( !bGif )
		WorkOnTokenSet( UNKNOWNCHAR, yytext );
	else
		WorkOnTokenSet( XML_RESSOURCEBUNDLE_END, yytext );
}

"<Context"[ \t]+"name"[ \t]*=[^\<]+\>	{
	if ( bGif )
		WorkOnTokenSet( UNKNOWNCHAR, yytext );
	else
		WorkOnTokenSet( XML_CONTEXT_START, yytext );
}

"<Template"[ \t]+"name"[ \t]*=[^\<]+\>	{
	if ( !bGif )
		WorkOnTokenSet( UNKNOWNCHAR, yytext );
	else
		WorkOnTokenSet( XML_CONTEXT_START, yytext );
}

"</Context>"	{
	if ( bGif )
		WorkOnTokenSet( UNKNOWNCHAR, yytext );
	else
		WorkOnTokenSet( XML_CONTEXT_END, yytext );
}

"</Template>"	{
	if ( !bGif )
		WorkOnTokenSet( UNKNOWNCHAR, yytext );
	else
		WorkOnTokenSet( XML_CONTEXT_END, yytext );
}

"<Key"[ \t]+"id"[ \t]*=[^\<]+\>	{
	if ( bGif )
		WorkOnTokenSet( UNKNOWNCHAR, yytext );
	else
		WorkOnTokenSet( XML_KEY_START, yytext );
}

"<OutputFile"[ \t]+"name"[ \t]*=[^\<]+\>	{
	if ( !bGif )
		WorkOnTokenSet( UNKNOWNCHAR, yytext );
	else
		WorkOnTokenSet( XML_KEY_START, yytext );
}

"</Key>"	{
	if ( bGif )
		WorkOnTokenSet( UNKNOWNCHAR, yytext );
	else
		WorkOnTokenSet( XML_KEY_END, yytext );
}

"</OutputFile>"	{
	if ( !bGif )
		WorkOnTokenSet( UNKNOWNCHAR, yytext );
	else
		WorkOnTokenSet( XML_KEY_END, yytext );
}

"<Text"[ \t]+"xml:lang"[ \t]*=[^\<]+\>	{
	bText = 1;
	WorkOnTokenSet( XML_TEXT_START, yytext );
}

"</Text>"	{
	bText = 0;
	WorkOnTokenSet( XML_TEXT_END, yytext );
}

"<Arg"[ \t]+"n"[ \t]*=[^\<]+\>	{
	if ( bText == 1 ) {
		WorkOnTokenSet( XML_TEXTCHAR, yytext );
		Argument( yytext );
	}
	else
		WorkOnTokenSet( UNKNOWNCHAR, yytext );
}

.|\n {
	if ( bText == 1 )
		WorkOnTokenSet( XML_TEXTCHAR, yytext );
	else
		WorkOnTokenSet( UNKNOWNCHAR, yytext );
}


%%

/*****************************************************************************/
int	yywrap(void)
/*****************************************************************************/
{
	return 1;
}

/*****************************************************************************/
void YYWarning( char *s )
/*****************************************************************************/
{
	/* write warning to stderr */
	fprintf( stderr,
		"Warning: \"%s\" in line %d: \"%s\"\n", s, yylineno, yytext  );
}

/*****************************************************************************/
#ifdef GCC
void yyerror ( char *s, ... )
#else
void yyerror ( char *s )
#endif
/*****************************************************************************/
{
	/* write error to stderr */
	fprintf( stderr,
		"Error: \"%s\" in line %d: \"%s\"\n", s, yylineno, yytext  );
	SetError();
}

/*****************************************************************************/
int
#ifdef WNT
_cdecl
#endif
main( int argc, char* argv[])
/*****************************************************************************/
{
	/* error level */
	int nRetValue = 0;
	char *pOutput;
	FILE *pFile;

	fprintf( stdout, "\nXmlEx 0.9 Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.\n" );
	fprintf( stdout, "======================================================================\n" );

	pOutput = GetOutputFile( argc, argv );
	bGif = IsXGF();

	if ( !pOutput ) {
		fprintf( stdout, "Syntax: XMLEX[-p Prj][-r PrjRoot]-i FileIn [-o FileOut][-m DataBase][-e][-b][-u][-d DoneFile][-g[:dtd]] [-t:(xgf|xrb)][-NOUTF8][-L l1,l2,...][-ISO99 IsoCode]\n" );
		fprintf( stdout, " Prj:      Project\n" );
		fprintf( stdout, " PrjRoot:  Path to project root (..\\.. etc.)\n" );
		fprintf( stdout, " FileIn:   Source files (*.src)\n" );
		fprintf( stdout, " FileOut:  Destination file (*.*)\n" );
		fprintf( stdout, " DataBase: Mergedata (*.sdf)\n" );
		fprintf( stdout, " -e: Disable writing errorlog\n" );
		fprintf( stdout, " -b: Break when Token \"HelpText\" found in source\n" );
		fprintf( stdout, " -u: [english] and [german] are allowed, Id is Taken from DataBase \n" );
		fprintf( stdout, " -g[:dtd]: enables generation of properties (dtds if :dtd is set) - in this case FileOut is the output path\n" );
		fprintf( stdout, " -d: enables generation of *.don if work is done\n" );
		fprintf( stdout, " -t(xgf|xrb): specifies the type of input file\n" );
		fprintf( stdout, " -NOUTF8: disable UTF8 as language independent encoding\n" );
		fprintf( stdout, " -L: Restrict the handled languages. l1,l2,... are elements of (01,33,46,49...)\n" );
		fprintf( stdout, "     A fallback language can be defined like this: l1=f1.\n" );
		fprintf( stdout, "     f1, f2,... are also elements of (01,33,46,49...)\n" );
		fprintf( stdout, "     Example: -L 01,99=35\n" );
		fprintf( stdout, "              Restriction to 01 and 99, 35 will be fallback for 99\n" );
		fprintf( stdout, " -ISO99: IsoCode is the full qualified ISO language code for language 99" );
		return 1;
	}

	pFile = GetXmlFile();
	if ( !pFile )
		return 1;

   	yyin = pFile;

	/* create global instance of class XmlExport */
	InitXmlExport( pOutput );

	/* start parser */
   	yylex();

	/* get error info. and end export */
	nRetValue = GetError();
	EndXmlExport();

	fprintf( stdout, "\n===================================\n\n" );

	/* return error level */
	return nRetValue;
}
